# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast
from functools import reduce

import numba
from numba import *
from numba import error
from numba import visitors, nodes
from numba import function_util
from numba.symtab import Variable
from numba.typesystem import is_obj
from numba import pyconsts

logger = logging.getLogger(__name__)

opmap = {
    ast.Eq        : pyconsts.Py_EQ,
    ast.NotEq     : pyconsts.Py_NE,
    ast.Lt        : pyconsts.Py_LT,
    ast.LtE       : pyconsts.Py_LE,
    ast.Gt        : pyconsts.Py_GT,
    ast.GtE       : pyconsts.Py_GE,
}

def build_boolop(right, left):
    node = ast.BoolOp(ast.And(), [left, right])
    return nodes.typednode(node, bool_)

def extract(complex_node):
    complex_node = nodes.CloneableNode(complex_node)

    real = nodes.ComplexAttributeNode(complex_node, 'real')
    imag = nodes.ComplexAttributeNode(complex_node.clone, 'imag')

    return real, imag

def compare(lhs, rhs):
    result = ast.Compare(lhs, [ast.Eq()], [rhs])
    return nodes.typednode(result, bool_)

class SpecializeComparisons(visitors.NumbaTransformer):
    """
    Rewrite cascaded ast.Compare nodes to a sequence of boolean operations
    ANDed together:

        a < b < c

    becomes

        a < b and b < c
    """

    def single_compare(self, node):
        rhs = node.comparators[0]

        if is_obj(node.left.type):
            node = self.single_compare_objects(node)

        elif node.left.type.is_pointer and rhs.type.is_pointer:
            # Coerce pointers to integer values before comparing
            node.left = nodes.CoercionNode(node.left, Py_uintptr_t)
            node.comparators = [nodes.CoercionNode(rhs, Py_uintptr_t)]

        elif node.left.type.is_complex and rhs.type.is_complex:
            real1, imag1 = extract(node.left)
            real2, imag2 = extract(rhs)
            lhs = compare(real1, real2)
            rhs = compare(imag1, imag2)
            result = ast.BoolOp(ast.And(), [lhs, rhs])
            node = nodes.typednode(result, bool_)

        return node

    def single_compare_objects(self, node):
        op = type(node.ops[0])
        if op not in opmap:
            raise error.NumbaError(
                    node, "%s comparisons not yet implemented" % (op,))

        # Build arguments for PyObject_RichCompareBool
        operator = nodes.const(opmap[op], int_)
        args = [node.left, node.comparators[0], operator]

        # Call PyObject_RichCompareBool
        compare = function_util.external_call(self.context,
                                              self.llvm_module,
                                              'PyObject_RichCompare',
                                              args=args)

        # Coerce int result to bool
        return nodes.CoercionNode(compare, node.type)

    def visit_Compare(self, node):
        "Reduce cascaded comparisons into single comparisons"

        # Process children
        self.generic_visit(node)

        compare_nodes = []
        comparators = [nodes.CloneableNode(c) for c in node.comparators]

        if len(node.comparators) > 1:
            if node.type.is_array:
                raise error.NumbaError(
                        node, "Cannot determine truth value of boolean array "
                              "(use any or all)")

        # Build comparison nodes
        left = node.left
        for op, right in zip(node.ops, comparators):
            node = ast.Compare(left=left, ops=[op], comparators=[right])

            # Set result type of comparison:
            #     bool array of array comparison
            #     bool otherwise

            if left.type.is_array or right.type.is_array:
                # array < x -> Array(bool_, array.ndim)
                result_type = self.context.promote_types(left.type, right.type)
            else:
                result_type = bool_

            nodes.typednode(node, result_type)

            # Handle comparisons specially based on their types
            node = self.single_compare(node)
            compare_nodes.append(node)

            left = right.clone

        # AND the comparisons together
        node = reduce(build_boolop, reversed(compare_nodes))

        return node



import ast

import numba
from numba import *
from numba import error
from numba import typesystem
from numba import visitors
from numba import nodes
from numba import function_util
from numba.exttypes import virtual
from numba.traits import traits, Delegate
from numba.typesystem import is_obj, promote_closest, promote_to_native

class ExtensionTypeLowerer(visitors.NumbaTransformer):
    """
    Lower extension type attribute accesses and method calls.
    """

    def get_handler(self, ext_type):
        if ext_type.is_jit_extension:
            return StaticExtensionHandler()
        else:
            return DynamicExtensionHandler()

    # ______________________________________________________________________
    # Attributes

    def visit_ExtTypeAttribute(self, node):
        """
        Resolve an extension attribute.
        """
        handler = self.get_handler(node.ext_type)
        self.visitchildren(node)
        return handler.handle_attribute_lookup(self.env, node)

    # ______________________________________________________________________
    # Methods

    def visit_NativeFunctionCallNode(self, node):
        if node.signature.is_bound_method:
            assert isinstance(node.function, nodes.ExtensionMethod)
            self.visitlist(node.args)
            node = self.visit_ExtensionMethod(node.function, node)
        else:
            self.visitchildren(node)

        return node

    def visit_ExtensionMethod(self, node, call_node=None):
        """
        Resolve an extension method. We currently only support immediate
        calls of extension methods.
        """
        if call_node is None:
            raise error.NumbaError(node, "Referenced extension method '%s' "
                                         "must be called" % node.attr)

        handler = self.get_handler(node.ext_type)
        return handler.handle_method_call(self.env, node, call_node)


#------------------------------------------------------------------------
# Handle Static VTable Attributes and Methods
#------------------------------------------------------------------------

class StaticExtensionHandler(object):
    """
    Handle attribute lookup and method calls for static extensions
    with C++/Cython-like virtual method tables and static object layouts.
    """

    def handle_attribute_lookup(self, env, node):
        """
        Resolve an extension attribute for a static object layout.

            ((attributes_struct *)
                 (((char *) obj) + attributes_offset))->attribute

        :node: ExtTypeAttribute AST node
        """
        ext_type = node.value.type
        offset = ext_type.attr_offset
        type = ext_type.attribute_table.to_struct()

        if isinstance(node.ctx, ast.Load):
            value_type = type.ref()         # Load result
        else:
            value_type = type.pointer()     # Use pointer for storage

        struct_pointer = nodes.value_at_offset(node.value, offset,
                                               value_type)
        result = nodes.StructAttribute(struct_pointer, node.attr,
                                       node.ctx, type.ref())

        return result

    def handle_method_call(self, env, node, call_node):
        """
        Resolve an extension method of a static (C++/Cython-like) vtable:

            typedef {
                double (*method1)(double);
                ...
            } vtab_struct;

            vtab_struct *vtab = *(vtab_struct **) (((char *) obj) + vtab_offset)
            void *method = vtab[index]
        """
        # Make the object we call the method on clone-able
        node.value = nodes.CloneableNode(node.value)

        ext_type = node.value.type
        offset = ext_type.vtab_offset

        vtable_struct = ext_type.vtab_type.to_struct()
        vtable_struct_type = vtable_struct.ref()

        vtab_struct_pointer_pointer = nodes.value_at_offset(
            node.value, offset,vtable_struct_type.pointer())
        vtab_struct_pointer = nodes.DereferenceNode(vtab_struct_pointer_pointer)

        vmethod = nodes.StructAttribute(vtab_struct_pointer, node.attr,
                                        ast.Load(), vtable_struct_type)

        # Insert first argument 'self' in args list
        args = call_node.args
        args.insert(0, nodes.CloneNode(node.value))
        result = nodes.NativeFunctionCallNode(node.type, vmethod, args)

        return result


#------------------------------------------------------------------------
# Handle Dynamic VTable Attributes and Methods
#------------------------------------------------------------------------

@traits
class DynamicExtensionHandler(object):
    """
    Handle attribute lookup and method calls for autojit extensions
    with dynamic perfect-hash-based virtual method tables and dynamic
    object layouts.
    """

    static_handler = StaticExtensionHandler()

    # TODO: Implement hash-based attribute lookup
    handle_attribute_lookup = Delegate('static_handler')

    def handle_method_call(self, env, node, call_node):
        """
        Resolve an extension method of a dynamic hash-based vtable:

            PyCustomSlots_Table ***vtab_slot = (((char *) obj) + vtab_offset)
            lookup_virtual_method(*vtab_slot)

        We may cache (*vtab_slot), but we may not cache (**vtab_slot), since
        compilations may regenerate the table.

        However, we could *preload* (**vtab_slot), where function calls
        invalidate the preload, if we were so inclined.
        """
        # Make the object we call the method on clone-able
        node.value = nodes.CloneableNode(node.value)

        ext_type = node.ext_type
        func_signature = node.type
        offset = ext_type.vtab_offset

        # __________________________________________________________________
        # Retrieve vtab

        vtab_ppp = nodes.value_at_offset(node.value, offset,
                                         void.pointer().pointer())
        vtab_struct_pp = nodes.DereferenceNode(vtab_ppp)

        # __________________________________________________________________
        # Calculate pre-hash

        prehash = virtual.hash_signature(func_signature, func_signature.name)
        prehash_node = nodes.ConstNode(prehash, uint64)

        # __________________________________________________________________
        # Retrieve method pointer

        # A method is always present when it was given a static signature,
        # e.g. @double(double)
        always_present = node.attr in ext_type.vtab_type.methodnames
        args = [vtab_struct_pp, prehash_node]

        # lookup_impl = NumbaVirtualLookup()
        lookup_impl = DebugVirtualLookup()
        ptr = lookup_impl.lookup(env, always_present, node, args)
        vmethod = ptr.coerce(func_signature.pointer())
        vmethod = vmethod.cloneable

        # __________________________________________________________________
        # Call method pointer

        # Insert first argument 'self' in args list
        args = call_node.args
        args.insert(0, nodes.CloneNode(node.value))
        method_call = nodes.NativeFunctionCallNode(func_signature, vmethod, args)

        # __________________________________________________________________
        # Generate fallback

        # TODO: Subclassing!
        # if not always_present:
        #     # TODO: Enable this path and generate a phi for the result
        #     # Generate object call
        #     obj_args = [nodes.CoercionNode(arg, object_) for arg in args]
        #     obj_args.append(nodes.NULL)
        #     object_call = function_util.external_call(
        #         env.context, env.crnt.llvm_module,
        #         'PyObject_CallMethodObjArgs', obj_args)
        #
        #     # if vmethod != NULL: vmethod(obj, ...)
        #     # else: obj.method(...)
        #     method_call = nodes.if_else(
        #         ast.NotEq(),
        #         vmethod.clone, nodes.NULL,
        #         lhs=method_call, rhs=object_call)

        return method_call

#------------------------------------------------------------------------
# Method lookup
#------------------------------------------------------------------------

def call_jit(jit_func, args):
    return nodes.NativeCallNode(jit_func.signature, args, jit_func.lfunc)

class NumbaVirtualLookup(object):
    """
    Use a numba function from numba.utility.virtuallookup to look up virtual
    methods in a hash table.
    """

    def lookup(self, env, always_present, node, args):
        """
        :param node: ExtensionMethodNode
        :param args: [vtable_node, prehash_node]
        :return: The virtual method as a Node
        """
        from numba.utility import virtuallookup

        if always_present and False:
            lookup = virtuallookup.lookup_method
        else:
            lookup = virtuallookup.lookup_and_assert_method
            args.append(nodes.const(node.attr, c_string_type))

        vmethod = call_jit(lookup, args)
        return vmethod

class DebugVirtualLookup(object):
    """
    Use a C utility function from numba/utility/utilities/virtuallookup.c
    to look up virtual methods in a hash table.

    Use for debugging.
    """

    def lookup(self, env, always_present, node, args):
        args.append(nodes.const(node.attr, c_string_type))
        vmethod = function_util.utility_call(
            env.context, env.crnt.llvm_module,
            "lookup_method", args)
        return vmethod



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast

from numba import visitors

class DeleteStatement(visitors.NumbaVisitor):
    """
    Delete a (compound) statement that contains basic blocks.
    The statement must be at the start of the entry block.

    idom: the immediate dominator of
    """

    def __init__(self, flow):
        self.flow = flow

    def visit_If(self, node):
        self.generic_visit(node)

        # Visit ControlBlocks
        self.visit(node.cond_block)
        self.visit(node.if_block)
        if node.orelse:
            self.visit(node.else_block)
        if node.exit_block:
            self.visit(node.exit_block)

    visit_While = visit_If

    def visit_For(self, node):
        self.generic_visit(node)

        # Visit ControlBlocks
        self.visit(node.cond_block)
        self.visit(node.if_block)
        self.visit(node.incr_block)
        if node.orelse:
            self.visit(node.else_block)
        if node.exit_block:
            self.visit(node.exit_block)


    def visit_ControlBlock(self, node):
        #print "deleting block", node
        for phi in node.phi_nodes:
            for incoming in phi.incoming:
                #print "deleting", incoming, phi
                incoming.cf_references.remove(phi)

        self.generic_visit(node)
        node.delete(self.flow)

    def visit_Name(self, node):
        references = node.variable.cf_references
        if isinstance(node.ctx, ast.Load) and node in references:
            references.remove(node)



from numba import *
from numba import register, register_callable, typeof, typeset

#----------------------------------------------------------------------------
# Type functions
#----------------------------------------------------------------------------

@register_callable(int32(double))
def func(arg):
    return int(arg)

@register_callable(typeset([int16(float_), int32(double), numeric(numeric)]))
def func_typeset_simple(arg):
    return int(arg)

@register_callable(numeric(numeric, numeric))
def func_typeset_binding(arg1, arg2):
    return int(arg1) + int(arg2)

#----------------------------------------------------------------------------
# Use of type functions
#----------------------------------------------------------------------------

@autojit
def use_user_type_function():
    return typeof(func(10.0))

@autojit
def use_typeset_function_simple():
    return typeof(func_typeset_simple(10.0))

@autojit
def use_typeset_function_binding(type1, type2):
    return typeof(func_typeset_binding(type1(10.0), type2(12.0)))

#----------------------------------------------------------------------------
# Test functions
#----------------------------------------------------------------------------

def test_register_callable():
    assert use_user_type_function() == int32
    assert use_typeset_function_simple() == int32

    assert use_typeset_function_binding(double, double) == double
    assert use_typeset_function_binding(float_, double) == double
    assert use_typeset_function_binding(int_, float_) == float_
    assert use_typeset_function_binding(int_, long_) == long_

if __name__ == '__main__':
    test_register_callable()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import llvm

def llvm_alloca(lfunc, builder, ltype, name='', change_bb=True):
    "Use alloca only at the entry bock of the function"
    if change_bb:
        bb = builder.basic_block
    builder.position_at_beginning(lfunc.get_entry_basic_block())
    lstackvar = builder.alloca(ltype, name)
    if change_bb:
        builder.position_at_end(bb)
    return lstackvar

def if_badval(translator, llvm_result, badval, callback,
              cmp=llvm.core.ICMP_EQ, name='cleanup'):
    # Use llvm_cbuilder :(
    b = translator.builder

    bb_true = translator.append_basic_block('%s.if.true' % name)
    bb_endif = translator.append_basic_block('%s.if.end' % name)

    test = b.icmp(cmp, llvm_result, badval)
    b.cbranch(test, bb_true, bb_endif)

    b.position_at_end(bb_true)
    callback(b, bb_true, bb_endif)
    # b.branch(bb_endif)
    b.position_at_end(bb_endif)

    return llvm_result



import os

from numba import *
from numba import error

@autojit(backend='ast', locals=dict(value=double))
def locals_override(obj):
    value = obj.method()
    with nopython:
        return value * value

class Class(object):
    def method(self):
        return 20.0

def test_locals_override():
    assert locals_override(Class()) == 400.0

if __name__ == "__main__":
    test_locals_override()



import os
import functools
from distutils.extension import Extension

import numpy as np

def prefix_module(prefix, module_name):
    if prefix:
        return "%s.%s" % (prefix, module_name)

    return module_name

def prefix_path(prefix, path):
    if prefix:
        return "%s/%s" % (prefix.rstrip("/"), path.lstrip("/"))

    return path

def make_extension(path_prefix, module_prefix, modname, sources, depends, **kwds):
    _prefix_path = functools.partial(prefix_path, path_prefix)

    return Extension(
        prefix_module(module_prefix, modname),
        sources=list(map(_prefix_path, sources)),
        depends=list(map(_prefix_path, depends)),
        **kwds
    )

def get_extensions(path_prefix, module_prefix=""):
    include_dirs = [prefix_path(path_prefix, 'include'),
                    np.get_include()]

    perfecthash_deps = ["include/perfecthash.h"]

    Extension = functools.partial(make_extension, path_prefix, module_prefix)

    extensions = [
        Extension("extensibletype.extensibletype",
                  ["extensibletype/extensibletype.pyx",
                   #'../ulib/src/base/md5sum.c',
                   #'../ulib/src/base/hash.c'
                  ],
                  include_dirs=include_dirs,
                  depends=perfecthash_deps),

        Extension("extensibletype.intern",
                  ["extensibletype/intern.pyx"],
                  include_dirs=include_dirs,
                  depends=["include/globalinterning.h",
                           "include/interning.h",
                           "include/perfecthash.h"]),

        Extension("extensibletype.methodtable",
                  ["extensibletype/methodtable.pyx"],
                  include_dirs=include_dirs,
                  depends=perfecthash_deps),

        Extension("extensibletype.test.pstdint",
                  ["extensibletype/test/pstdint.pyx"],
                  include_dirs=include_dirs,
                  depends=["include/pstdint.h"]),
    ]

    return extensions



cdef double nan

from numpy import nan
cimport cython
from customslots cimport *

cdef extern from "thestandard.h":
    unsigned long EXTENSIBLETYPE_DOUBLE_FUNC_SLOT

# Import call_func from a seperately compiled C file to avoid inlining it

ctypedef double (*funcptr_t)(double)
cdef extern:
    double call_func(object)

def sum_lookups(obj, int n):
    # Do the lookup n times
    cdef int i
    cdef double s = 0
    for i in range(n):
        s += call_func(obj)
    return s

def sum_baseline(obj, int n):
    # Do the same work as sum_lookups, but only do the lookup once
    cdef double s = 0
    cdef PyCustomSlot *slot = PyCustomSlots_Find(
            obj, EXTENSIBLETYPE_DOUBLE_FUNC_SLOT, 0)
    if slot == NULL:
        return 0.0
    cdef funcptr_t funcptr = <funcptr_t>slot.data.pointer
    for i in range(n):
        s += funcptr(3.0)
    return s



"""
>>> a = np.arange(10, dtype=np.double)

>>> preload_arg(a)
4.0
>>> preload_local(a)
4.0
>>> preload_phi(a)
45.0
>>> preload_phi_cycle(a)
45.0
>>> preload_phi_cycle2(a)
45.0
"""

import numpy as np
from numba import *

@autojit
def preload_arg(A):
    return A[4]

@autojit
def preload_local(A):
    A = A
    return A[4]

@autojit
def preload_phi(A):
    sum = 0.0
    for i in range(10):
        sum += A[i]
        A = A

    return sum

@autojit
def preload_phi_cycle(A):
    # A_0 = A                       # <- propagated preload
    sum = 0.0
    for i in range(10):
        # A_1 = phi(A_0, A_3)       # <- preload
        sum += A[i]
        if i > 5:
            A = A # A_2             # <- propagated preload
        # A_3 = phi(A_1, A_2)       # <- propagated preload

    return sum

@autojit
def preload_phi_cycle2(A):
    # A_0 = A                       # <- propagated preload
    sum = 0.0
    for i in range(10):
        # A_1 = phi(A_0, A_3)       # <- propagated preload
        if i > 5:
            A = A # A_2             # <- propagated preload

        # A_3 = phi(A_1, A_2)       # <- preload
        sum += A[i]


    return sum

if __name__ == "__main__":
#    a = np.arange(10, dtype=np.double)
#    preload_arg(a)
#    preload_phi(a)
    import numba
    numba.testing.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import fnmatch

from numba.typesystem.basetypes import *

def _typematch(pattern, typerepr):
    return fnmatch.fnmatch(typerepr, pattern)

def typematch(pattern, ty):
    """
    Match a type pattern to a type.

    >>> type = ListType(object_, 2)
    >>> typematch("list(*, 2)", type)
    True
    >>> typematch("list(*)", type)
    True
    >>> typematch("list(*)", type)
    True
    >>> typematch("tuple(*)", type)
    False
    >>> typematch("object_", type)
    True
    """
    return any(_typematch(pattern, cls.__repr__(ty))
                   for cls in type(ty).__mro__)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



cimport cython
from numba._numba cimport *
cimport numpy as cnp

import types
import ctypes

import numba
from numba import error
from numba.minivect import minitypes
from numba.support import ctypes_support, cffi_support

import numpy as np


cdef extern from *:
    ctypedef struct PyTypeObject:
        PyObject *tp_dict

    ctypedef struct PyMethodDef:
        pass

cdef extern from "numbafunction.h":
    cdef size_t closure_field_offset
    cdef PyTypeObject *NumbaFunctionType
    cdef int NumbaFunction_init() except -1
    cdef object NumbaFunction_NewEx(
            PyMethodDef *ml, module, code, PyObject *closure,
            void *native_func, native_signature, keep_alive)

#------------------------------------------------------------------------
# Numba Function Wrappers
#------------------------------------------------------------------------

cdef class NumbaWrapper(object):
    """
    Numba wrapper function.

        py_func: original Python function
    """

    cdef public object py_func
    cdef public object func_name, func_doc, module

    def __init__(self, py_func):
        self.py_func = py_func

        self.func_name = py_func.__name__
        self.func_doc = py_func.__doc__
        self.module = py_func.__module__

#------------------------------------------------------------------------
# Create Numba Functions (numbafunction.c)
#------------------------------------------------------------------------

NumbaFunction_init()
NumbaFunction_NewEx_pointer = <Py_uintptr_t> &NumbaFunction_NewEx

numbafunction_type = <object> NumbaFunctionType
numbafunc_closure_field_offset = closure_field_offset

def create_function(methoddef, py_func, lfunc_pointer, signature, modname):
    cdef Py_uintptr_t methoddef_p = ctypes.cast(ctypes.byref(methoddef),
                                                ctypes.c_void_p).value
    cdef PyMethodDef *ml = <PyMethodDef *> methoddef_p
    cdef Py_uintptr_t lfunc_p = lfunc_pointer

    result = NumbaFunction_NewEx(ml, modname, getattr(py_func, "__code__", None),
                                 NULL, <void *> lfunc_p, signature, py_func)
    return result

#------------------------------------------------------------------------
# Classes to exclude from type hashing
#------------------------------------------------------------------------

support_classes = (ctypes_support.CData,)
if cffi_support.ffi is not None:
    support_classes += (cffi_support.cffi_func_type,)

cdef tuple hash_on_value_types = (
    minitypes.Type,
    np.ufunc,
    np.dtype,
    np.generic,
    NumbaWrapper,
    types.FunctionType,
    types.BuiltinFunctionType,
    types.MethodType,
    getattr(types, 'UnboundMethodType', types.FunctionType),
    types.BuiltinMethodType,
) # + support_classes

def add_hash_by_value_type(type):
    global hash_on_value_types

    hash_on_value_types += (type,)

#------------------------------------------------------------------------
# @jit function creation
#------------------------------------------------------------------------

cdef class NumbaCompiledWrapper(NumbaWrapper):
    """
    Temporary numba wrapper function for @jit, only used for recursion.

        signature: minitype FunctionType signature
        lfunc: LLVM function
    """

    cdef public object lfunc, signature, wrapper, lfunc_pointer

    def __init__(self, py_func, signature, lfunc):
        super(NumbaCompiledWrapper, self).__init__(py_func)

        self.signature = signature
        self.lfunc = lfunc
        self.lfunc_pointer = None

    def __repr__(self):
        return '<compiled numba function (%s) :: %s>' % (self.py_func,
                                                         self.signature)

def create_numba_wrapper(py_func, numbafunction, signature, lfunc):
    """
    Use the NumbaFunction to set attributes for numba, and return the
    NumbaFunction.
    """
    if numbafunction is None:
        # Function is called recursively, use a placeholder
        return NumbaCompiledWrapper(py_func, signature, lfunc)

    numbafunction.py_func = py_func
    numbafunction.signature = signature
    numbafunction.lfunc = lfunc
    return numbafunction

def is_numba_wrapper(numbafunction):
    """
    Check whether the given object is a numba function wrapper around a
    compiled function.
    """
    return isinstance(numbafunction, (NumbaCompiledWrapper, numbafunction_type))

#------------------------------------------------------------------------
# Numba Autojit Function Wrapper
#------------------------------------------------------------------------

cdef class _NumbaSpecializingWrapper(NumbaWrapper):
    """
    Numba wrapper function for @autojit. Specializes py_func when called with
    new argtypes.

        py_func: original Python function
        compiler: numba.wrapper.compiler.Compiler
                  Compiles the function given the argument values.
        funccache: AutojitFunctionCache that can quickly lookup the right
                   specialization
    """

    cdef public AutojitFunctionCache funccache
    cdef public object compiler

    def __init__(self, py_func, compiler, funccache):
        super(_NumbaSpecializingWrapper, self).__init__(py_func)
        self.compiler = compiler
        self.funccache = funccache

    def __repr__(self):
        return '<specializing numba function(%s)>' % self.py_func

    def add_specialization(self, signature):
        numba_wrapper = self.compiler.compile(signature)

        # NOTE: We do not always have args (e.g. if one autojit function
        # or method calls another one). However, the first call from Python
        # will retrieve it from the slow cache (env.specializations)

        # self.funccache.add(args, numba_wrapper)

        return numba_wrapper

    def __call__(self, *args, **kwargs):
        if len(kwargs):
            raise error.NumbaError("Cannot handle keyword arguments yet")

        numba_wrapper = self.funccache.lookup(args)
        if numba_wrapper is None:
            # print "Cache miss for function:", self.py_func.__name__
            numba_wrapper = self.compiler.compile_from_args(args, kwargs)
            self.funccache.add(args, numba_wrapper)

        return PyObject_Call(<PyObject *> numba_wrapper,
                             <PyObject *> args, NULL)

    def __get__(self, instance, type):
        if instance is None:
            if numba.PY3:
                return UnboundFunctionWrapper(self, type)
            else:
                return self

        return BoundSpecializingWrapper(self, instance)


class NumbaSpecializingWrapper(_NumbaSpecializingWrapper):
    # Don't make this a docstring, it breaks the __doc__ propertyr
    # """
    # Python class to allow overriding properties such as __name__.
    # """

    @property
    def __name__(self):
        return self.func_name

    @property
    def __doc__(self):
        return self.func_doc

    @property
    def __module__(self):
        return self.module


#------------------------------------------------------------------------
# Unbound Methods
#------------------------------------------------------------------------

def unbound_method_type_check(py_class, obj):
    if not isinstance(obj, py_class):
        raise TypeError(
            "unbound method numba_function_or_method object must be "
            "called with %s instance as first argument "
            "(got %s instance instead)" % (py_class.__name__, type(obj).__name__))


cdef class UnboundFunctionWrapper(object):
    """
    Wraps unbound functions in Python 3, for jit and autojit methods.
    PyInstanceMethod does not check whether 'self' is an instance of 'type'.

    Hence the following works in Python 3:

        class C(object):
            def m(self):
                print self

        C.m(object())

    However, this is dangerous for numba code, since it expects an instance
    of 'C' (and may access fields in a low-level way).
    """

    cdef public object func, type

    def __init__(self, func, type):
        self.func = func
        self.type = type

    def __call__(self, *args, **kwargs):
        assert len(args) > 0, "Unbound method must have at least one argument"
        unbound_method_type_check(self.type, args[0])
        return self.func(*args, **kwargs)


cdef public Create_NumbaUnboundMethod(PyObject *func, PyObject *type):
    assert type != NULL
    obj = UnboundFunctionWrapper(<object> func, <object> type)
    return obj

#------------------------------------------------------------------------
# Bound Methods
#------------------------------------------------------------------------

cdef class BoundSpecializingWrapper(object):
    """
    Numba wrapper created for bound @autojit methods. Note that @jit methods
    don't need this, since numbafunction does the binding.
    """

    cdef public object specializing_wrapper, type, instance

    def __init__(self, specializing_wrapper, instance):
        self.specializing_wrapper = specializing_wrapper
        self.instance = instance

    def __call__(self, *args, **kwargs):
        return self.specializing_wrapper(self.instance, *args, **kwargs)

#------------------------------------------------------------------------
# Autojit Fast Function Cache
#------------------------------------------------------------------------

cdef inline _id(obj):
    return <Py_uintptr_t> <PyObject *> obj

cdef inline void setkey(t, int i, k):
    Py_INCREF(<PyObject *> k)
    PyTuple_SET_ITEM(t, i, k)

cpdef inline getkey(tuple args): # 3.0x
    """
    Get the tuple key we need to look up the right specialization from the
    runtime autojit arguments.

    We micro-optimize this to avoid significant overhead for short functions
    (the dispatch may in fact be significantly more expensive than the actual
     function call).
    """
    cdef Py_ssize_t i

    cdef Py_ssize_t nargs = PyTuple_GET_SIZE(args)
    cdef tuple key = PyTuple_New(nargs * 3)
    cdef cnp.ndarray array

    for i in range(nargs):
        arg = <object> PyTuple_GET_ITEM(args, i)
        # arg = args[i]
        if isinstance(arg, cnp.ndarray):
            # Also specialize of C/F data when we implement optimizations
            # for those.
            array = <cnp.ndarray> arg

            # Hashing on dtype here makes the test example more than
            # twice as slow, hash on its id() instead.
            # k = (type(arg), array.descr, array.ndim)
            # k = (type(arg), _id(array.descr), array.ndim)
            setkey(key, i*3, type(arg))
            setkey(key, i*3+1, _id(array.descr))
            # NumPy maximum ndim is 32 (2 ** 5).
            setkey(key, i*3+2, array.ndim | (cnp.PyArray_FLAGS(arg) << 5))
            continue

        elif isinstance(arg, hash_on_value_types):
            # A type is passed in as a value, hash the thing (this will be slow)
            setkey(key, i*3, arg)

        else:
            # k = type(arg)
            setkey(key, i*3, type(arg))

        setkey(key, i*3+1, 0)
        setkey(key, i*3+2, 0)

        # Py_INCREF(<PyObject *> k)
        # PyTuple_SET_ITEM(key, i, k)

    return key


cdef class AutojitFunctionCache(object):
    """
    Try a faster lookup for autojit functions.

    This function cache may give none where a compiled specialization does
    exist. This is caught by the slow path going through
    functions.FunctionCache.
    """

    cdef public dict specializations

    # list of dtypes that need to be alive in order for the id() hash to
    # remain valid
    cdef list dtypes

    def __init__(self):
        self.specializations = {}
        self.dtypes = []

    cpdef add(self, args, wrapper):
        # self.specializations[0] = wrapper
#        key = (0x19228, 0x384726)
        key = getkey(args)
        self.specializations[key] = wrapper

        for arg in args:
            if isinstance(arg, np.ndarray):
                self.dtypes.append(arg.dtype)

    cdef lookup(self, tuple args):
        # return self.specializations[0]
        key = getkey(args)

#        key = (0x19228, 0x384726)
        wrapper = self.specializations.get(key)
        return wrapper



# -*- coding: utf-8 -*-
"""
Adapted from Cython/Compiler/Visitor.py, see this module for detailed
explanations.
"""
from __future__ import print_function, division, absolute_import

import inspect

miniast = None # avoid circular import AttributeError for sphinx-apidoc
from . import treepath

class TreeVisitor(object):
    """
    Non-mutating visitor. Subclass and implement visit_MyNode methods.
    A user can traverse a foreign AST by implementing
    :py:class:`minivect.miniast.Context.getchildren`
    """

    want_access_path = False

    def __init__(self, context):
        self.context = context
        self.dispatch_table = {}
        if self.want_access_path:
            self.access_path = []
        else:
            self._visitchild = self.visit

    def _find_handler(self, obj):
        # to resolve, try entire hierarchy
        cls = type(obj)
        pattern = "visit_%s"
        mro = inspect.getmro(cls)
        handler_method = None
        for mro_cls in mro:
            handler_method = getattr(self, pattern % mro_cls.__name__, None)
            if handler_method is not None:
                return handler_method

        raise RuntimeError("Visitor %r does not accept object: %s" % (self, obj))

    def visit(self, obj, *args):
        "Visit a single child."
        try:
            handler_method = self.dispatch_table[type(obj)]
        except KeyError:
            handler_method = self._find_handler(obj)
            self.dispatch_table[type(obj)] = handler_method
        return handler_method(obj)

    def _visitchild(self, child, parent, attrname, idx):
        self.access_path.append((parent, attrname, idx))
        result = self.visit(child)
        self.access_path.pop()
        return result

    def visit_childlist(self, child, parent=None, attr=None):
        if isinstance(child, list):
            childretval = [self._visitchild(child_node, parent, attr, idx)
                               for idx, child_node in enumerate(child)]
        else:
            childretval = self._visitchild(child, parent, attr, None)
            if isinstance(childretval, list):
                raise RuntimeError(
                    'Cannot insert list here: %s in %r' % (attr, node))

        return childretval

    def visitchildren(self, parent, attrs=None):
        "Visits the children of the given node."
        if parent is None:
            return None

        if attrs is None:
            attrs = self.context.getchildren(parent)

        result = {}
        for attr in attrs:
            child = getattr(parent, attr)
            if child is not None:
                result[attr] = self.visit_childlist(child, parent, attr)

        return result

    def treepath(self, node, xpath_expr):
        return treepath.iterfind(node, xpath_expr)

    def treepath_first(self, node, xpath_expr):
        return treepath.find_first(node, xpath_expr)

    def p(self, node):
        node.print_tree(self.context)

class VisitorTransform(TreeVisitor):
    """
    Mutating transform. Each attribute is replaced by the result of the
    corresponding visit_MyNode method.
    """

    def visitchildren(self, parent, attrs=None):
        result = super(VisitorTransform, self).visitchildren(parent, attrs)
        for attr, newnode in result.iteritems():
            if not isinstance(newnode, list):
                setattr(parent, attr, newnode)
            else:
                # Flatten the list one level and remove any None
                newlist = []
                for x in newnode:
                    if x is not None:
                        if isinstance(x, list):
                            newlist += x
                        else:
                            newlist.append(x)
                setattr(parent, attr, newlist)
        return result

class GenericVisitor(TreeVisitor):
    "Generic visitor that automatically visits children"

    def visit_Node(self, node):
        self.visitchildren(node)
        return node

class GenericTransform(VisitorTransform, GenericVisitor):
    "Generic transform that automatically visits children"

class MayErrorVisitor(TreeVisitor):
    """
    Determine whether code generated by an AST can raise exceptions.
    """

    may_error = False

    def visit_Node(self, node):
        self.visitchildren(node)

    def visit_NodeWrapper(self, node):
        self.may_error = (self.may_error or
                          self.context.may_error(node.opaque_node))

    def visit_ForNode(self, node):
        self.visit(node.init)
        self.visit(node.condition)
        self.visit(node.step)

class PrintTree(TreeVisitor):
    """
    Print an AST, see also :py:class:`minivect.miniast.Node.print_tree`.
    """

    indent = 0
    want_access_path = True

    def format_value(self, node):
        from . import miniast

        if node.is_temp:
            format_value = node.repr_name
        elif (isinstance(node, miniast.Variable) or
              isinstance(node, miniast.FuncNameNode) or
              node.is_funcarg):
            format_value = node.name
        elif node.is_binop or node.is_unop:
            format_value = node.operator
        elif node.is_constant:
            format_value = node.value
        elif node.is_sizeof:
            format_value = str(node.type)
        else:
            return None

        return format_value

    def format_node(self, node, want_type_info=True):
        result = type(node).__name__
        format_value = self.format_value(node)

        if node.is_expression and want_type_info:
            if format_value is not None:
                format_value = "%s, type=%s" % (format_value, node.type)
            else:
                format_value = "type=%s" % (node.type,)

        if format_value:
            return "%s(%s)" % (result, format_value)
        else:
            return result

    def visit_Node(self, node):
        if self.access_path:
            parent, attr, idx = self.access_path[-1]
        else:
            attr = "(root)"
            idx = None

        prefix = "%s%s" % (self.indent * "  ", attr)
        if idx is not None:
            prefix = "%s[%d]" % (prefix, idx)

        print(("%s: %s" % (prefix, self.format_node(node))))

        self.indent += 1
        self.visitchildren(node)
        self.indent -= 1



# -*- coding: utf-8 -*-
"""
Support for CFFI. Allows checking whether objects are CFFI functions and
obtaining the pointer and numba signature.
"""
from __future__ import print_function, division, absolute_import

from numba import *
from numba.minivect.minitypes import *
from numba.minivect import minitypes, minierror

try:
    import cffi
    ffi = cffi.FFI()
except ImportError:
    ffi = None

def is_cffi_func(obj):
    "Check whether the obj is a CFFI function"
    try:
        return type(obj) is cffi_func_type
        # This is dangerous:
        #   >>> ffi.typeof("void (*)()")
        #   <ctype 'void(*)()'>
        # return ffi.typeof(obj).kind == 'function'
    except TypeError:
        return False

def get_pointer(cffi_func):
    """
    Get a pointer to the underlying function for a CFFI function as an
    integer.
    """
    return int(ffi.cast("uintptr_t", cffi_func))

def map_type(cffi_type):
    "Map CFFI type to numba type"
    if cffi_type.kind in ('struct', 'union'):
        if cffi_type.kind == 'union':
            result = None
        else:
            result = struct([(name, map_type(field_type))
                               for name, field_type in cffi_type.fields])
    elif cffi_type.kind == 'function':
        restype = map_type(cffi_type.result)
        argtypes = [map_type(arg) for arg in cffi_type.args]
        result = minitypes.FunctionType(restype, argtypes,
                                        is_vararg=cffi_type.ellipsis).pointer()
    else:
        result = type_map.get(cffi_type)

    if result is None:
        raise minierror.UnmappableTypeError(cffi_type)

    return result

def get_signature(cffi_func):
    "Get the numba signature for a CFFI function"
    return map_type(ffi.typeof(cffi_func)).base_type

if ffi is None:
    # Disable cffi support
    is_cffi_func = lambda x: False
    type_map = None
else:
    type_map = {
        ffi.typeof('char') :                char,
        ffi.typeof('short') :               short,
        ffi.typeof('int') :                 int_,
        ffi.typeof('long') :                long_,
        ffi.typeof('long long') :           longlong,
        ffi.typeof('unsigned char') :       uchar,
        ffi.typeof('unsigned short') :      ushort,
        ffi.typeof('unsigned int') :        uint,
        ffi.typeof('unsigned long') :       ulong,
        ffi.typeof('unsigned long long') :  ulonglong,
        ffi.typeof('float') :               float_,
        ffi.typeof('double') :              double,
        ffi.typeof('long double') :         longdouble,
        ffi.typeof('char *') :              c_string_type,
        ffi.typeof('ssize_t') :             Py_ssize_t,
        ffi.typeof('size_t') :              size_t,
    }

    ffi.cdef("int printf(char *, ...);")
    lib = ffi.dlopen(None)
    cffi_func_type = type(lib.printf)



cdef extern from "customslots.h":
    ctypedef unsigned long uintptr_t

    ctypedef union pyx_data:
        void *pointer
        Py_ssize_t objoffset
        uintptr_t flags

    ctypedef struct PyCustomSlot:
        uintptr_t id
        pyx_data data

    int PyCustomSlots_Check(obj)
    Py_ssize_t PyCustomSlots_Count(obj)
    PyCustomSlot *PyCustomSlots_Table(obj)

    PyCustomSlot *PyCustomSlots_Find(obj, uintptr_t id, Py_ssize_t start)



# -*- coding: utf-8 -*-
"""
Defines the typeset class and a number of builtin type sets.
"""
from __future__ import print_function, division, absolute_import

import collections
from functools import reduce
from itertools import starmap

from itertools import izip

from numba.typesystem import basetypes
from numba.minivect import minitypes

__all__ = [ 'typeset', 'numeric', 'integral', 'floating', 'complextypes' ]

#----------------------------------------------------------------------------
# Signature matching
#----------------------------------------------------------------------------

def _match_argtype(type1, type2):
    return (type1.is_typeset and type2 in type1.types) or type1 == type2

def _build_position_table(signature):
    table = collections.defaultdict(list)
    for i, argtype in enumerate(signature.args):
        if argtype.is_typeset:
            table[argtype].append(i)

    return table

def get_effective_argtypes(context, signature, argtypes):
    """
    Get promoted argtypes for typeset arguments, e.g.

        signature = floating(floating, floating)
        argtypes = [float, double]

    =>

        [double, double]
    """
    args = list(argtypes)
    position_table = _build_position_table(signature)
    promotion_table = {}

    for poslist in position_table.values():
        if len(poslist) > 1:
            # Find all argument types corresponding to a type set
            types = [args[i] for i in poslist]

            # Promote corresponding argument types
            result_type = reduce(context.promote_types, types)

            # Update promotion table
            type_set = signature.args[poslist[-1]]
            promotion_table[type_set] = result_type

            # Build coherent argument type list
            for i in poslist:
                args[i] = result_type

    return promotion_table, args

def match(context, signature, argtypes):
    """
    See whether a specialization matches the given function signature.
    """
    if len(signature.args) == len(argtypes):
        promotion_table, args = get_effective_argtypes(
                            context, signature, argtypes)

        if all(starmap(_match_argtype, izip(signature.args, args))):
            restype = signature.return_type
            restype = promotion_table.get(restype, restype)
            return restype(*args)

    return None


#----------------------------------------------------------------------------
# Type sets
#----------------------------------------------------------------------------

class typeset(minitypes.Type):
    """
    Holds a set of types that can be used to specify signatures for
    type inference.
    """

    is_typeset = True

    def __init__(self, types, name=None):
        super(typeset, self).__init__()

        self.types = frozenset(types)
        self.name = name
        self.first_type = types[0]

        self._from_argtypes = {}
        for type in types:
            if type.is_function:
                self._from_argtypes[type.args] = type

    def find_match(self, context, argtypes):
        argtypes = tuple(argtypes)
        if argtypes in self._from_argtypes:
            return self._from_argtypes[argtypes]

        for type in self.types:
            signature = match(context, type, argtypes)
            if signature:
                return signature

        return None

    def __iter__(self):
        return iter(self.types)

    def __repr__(self):
        return "typeset(%s, ...)" % (self.first_type,)

    def __hash__(self):
        return hash(id(self))


numeric = typeset(minitypes.numeric)
integral = typeset(minitypes.integral)
floating = typeset(minitypes.floating)
complextypes = typeset(minitypes.complextypes)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# ______________________________________________________________________
'''scrape_multiarray_api

Utilities for reading the __multiarray_api.h file, and scraping three
things: symbolic names of API members, array indices for those
members, and LLVM types for those members.
'''
from __future__ import print_function, division, absolute_import
# ______________________________________________________________________

import sys

import pprint

# ______________________________________________________________________

TEMPLATE_STR = '''"""Automatically generated code.

Automatically generated by .../numba/scrape_multiarray_api.py

Edit at your own risk!
"""

import llvm.core as lc
from .llvm_types import _int1, _int8, _int32, _int64, _intp, _numpy_struct, \\
     _numpy_array

API_INDEX_MAP = %(indexmap)s

API_TYPE_MAP = %(tymap)s

# End of automatically generated code.
'''

# ______________________________________________________________________

def joinlines (source_lines):
    '''Remove backslashes appearing next to line breaks in a string
    and strip it after doing so.'''
    return ''.join((ln[:-1] if ln[-1] == '\\' else ln
                    for ln in source_lines.splitlines())).strip()

# ______________________________________________________________________

TY_MAP = {
    'char' : '_int8',
    'int' : '_int32', # This seems to hold true, even on 64-bit systems.
    'unsigned char' : '_int8', # XXX
    'unsigned int' : 'u_int32', # XXX
    'void' : 'lc.Type.void()',
    'npy_bool' : '_int8',
    'npy_intp' : '_intp',
    'npy_uint32' : 'u_int32', # XXX/Note: Loses unsigned info in LLVM type.
    'PyArrayObject' : '_numpy_struct',
    'double' : 'lc.Type.double()',
    'size_t' : 'u_intp', # XXX Loses unsigneded-ness
    'npy_int64' : '_int64',
    'npy_datetime' : '_int64', # npy_common.h
    'npy_timedelta' : '_int64', # npy_common.h
}

# ______________________________________________________________________

def map_type (ty_str):
    npointer = ty_str.count('*')
    if npointer == 0:
        base_ty = ty_str
    else:
        base_ty = ty_str[:-npointer].strip()
    if base_ty == 'void' and npointer > 0:
        base_ty = '_int8'
    elif base_ty not in TY_MAP:
        if npointer > 0:
            base_ty = '_int8' # Basically cast into void *
        else:
            base_ty = '_int32' # Or an int.
    else:
        base_ty = TY_MAP[base_ty]
        if base_ty == '_numpy_struct' and npointer > 0:
            base_ty = '_numpy_array'
            npointer -= 1
    return ''.join((npointer * 'lc.Type.pointer(', base_ty, ')' * npointer))

# ______________________________________________________________________

def c_ty_str_to_llvm (c_ty_str):
    ty_str_fn_split = [substr.strip() for substr in c_ty_str.split('(*)')]
    ret_val = map_type(ty_str_fn_split[0])
    if len(ty_str_fn_split) > 1:
        arg_ty_strs = ty_str_fn_split[1][1:-1].split(', ')
        if len(arg_ty_strs) == 1 and arg_ty_strs[0].strip() == 'void':
            arg_ty_strs = []
        ret_val = ('lc.Type.function(%s, [%s])' %
                   (ret_val, ', '.join((map_type(arg_ty_str.strip())
                                        for arg_ty_str in arg_ty_strs))))
    return ret_val

# ______________________________________________________________________

def process_type (ty_str):
    if ty_str.startswith('(*'):
        ty_str = ty_str[3:-1]
    else:
        assert ty_str[0] == '('
        ty_str = ty_str[2:-1]
    return ty_str

# ______________________________________________________________________

def process_definition (source_defn):
    arr_str = 'PyArray_API['
    arr_str_idx = source_defn.index(arr_str)
    arr_str_end_idx = source_defn.index('])', arr_str_idx)
    return (int(source_defn[arr_str_idx + len(arr_str):arr_str_end_idx]),
            process_type(joinlines(source_defn[:arr_str_idx].strip())))

# ______________________________________________________________________

def process_source (source_file_path):
    ret_val = None
    with open(source_file_path) as source_file:
        source_text = source_file.read()
    split_by_pp_define0 = source_text.split('#define ')
    split_by_pp_define1 = (substr0.strip().split(None, 1)
                           for substr0 in split_by_pp_define0)
    return dict(((sym_name, process_definition(sym_defn))
                 for sym_name, sym_defn in split_by_pp_define1
                 if (sym_name != 'PyArray_API' and
                     sym_defn.find('])') != -1)))

# ______________________________________________________________________

def gen_python (processed_source, template_str = None):
    if template_str is None:
        template_str = TEMPLATE_STR
    index_map = {}
    ty_map_strs = ['{']
    for symbol, (index, c_ty_str) in processed_source.iteritems():
        index_map[symbol] = index
        ty_map_strs.append(' %r : %s,' % (symbol, c_ty_str_to_llvm(c_ty_str)))
    ty_map_strs.append('}')
    return template_str % {'indexmap' : pprint.pformat(index_map),
                           'tymap' : '\n'.join(ty_map_strs)}

# ______________________________________________________________________

def get_include ():
    import os, numpy
    return os.path.join(numpy.get_include(), 'numpy', '__multiarray_api.h')

# ______________________________________________________________________

def main (*args, **kws):
    '''Initial prototype for automatically generating multiarray C API
    call information for llvm-py.  Not actually used to generate any
    Numba modules (dynamically handled by multiarray_api).'''
    if len(args) == 0:
        args = (get_include(),)
    for arg in args:
        print((gen_python(process_source(arg))))

# ______________________________________________________________________

if __name__ == "__main__":
    import sys
    main(*sys.argv[1:])

# ______________________________________________________________________
# End of scrape_multiarray_api.py



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import numpy as np
from numba import autojit, jit, double, void, int32

@autojit
def limiter(x, n):
    if x + 1 > n:
        return n
    else:
        return x + 1

@jit(void(double[:,:,:], double[:,:,:], double[:,:,:], int32, int32, int32))
def gather_convolution_core(A, B, C, x, y, z):
    dimA = A.shape[0]
    bx = limiter(x, dimA)
    by = limiter(y, dimA)
    bz = limiter(z, dimA)
    for x1 in range(bx):
        for x2 in range(bx):
            if x1 + x2 == x:
                for y1 in range(by):
                    for y2 in range(by):
                        if y1 + y2 == y:
                            for z1 in range(bz):
                                for z2 in range(bz):
                                    if z1 + z2 == z:
                                        C[x, y, z] += A[x1, y1, z1] * B[x2, y2, z2]


def gather_convolution(A, B, C):
    from itertools import product
    dimC = C.shape[0]
    for x, y, z in product(range(dimC), range(dimC), range(dimC)):
        gather_convolution_core(A, B, C, x, y, z)

def test_convolution():
    # Creating some fake data to test this problem
    s = 4
    array_a = np.random.rand(s ** 3).reshape(s, s, s)
    array_b = np.random.rand(s ** 3).reshape(s, s, s)

    dimA = array_a.shape[0]
    dimB = array_b.shape[0]
    dimC = dimA + dimB

    array_c = np.zeros((dimC, dimC, dimC))

    gather_convolution(array_a, array_b, array_c)

if __name__ == '__main__':
    test_convolution()




"""
>>> empty_str()
''
>>> str_convert(12.2)
'12.2'
"""

import sys

from numba import *

@autojit(backend='ast')
def empty_str():
    x = str()
    return x

@autojit(backend='ast')
def str_convert(x):
    return str(x)

if __name__ == '__main__':
    import numba
    numba.testing.testmod()



from numba import *

@jit(void())
def func1():
    "I am a docstring!"

@autojit
def func2():
    "I am a docstring!"


if __name__ == '__main__':
    assert func1.__name__ == func1.py_func.__name__
    assert func1.__doc__ == "I am a docstring!"
    assert func1.__module__ == func1.py_func.__module__

    assert func2.__name__ == func2.py_func.__name__
    assert func2.__doc__ == "I am a docstring!", func2.__doc__
    # This does not yet work for some reason (maybe overridden by PyType_Ready()?)
    # assert func2.__module__ == func2.py_func.__module__



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast, sys

def prettyprint(node, stream=sys.stdout):
    text = ast.dump(node)
    depth = 0
    last = ''
    for i in text:
        if i == ' ':
            continue # ignore space

        indent = ' ' * 4 * depth
        if last in ['(', '[', ','] and i not in [')', ']']:
            stream.write('\n' + indent)

        if i in ['(', '[']:
            depth += 1
        elif i in [')', ']']:
#            if last not in ['(', '[']:
#                stream.write('\n' + indent)
            depth -= 1

        stream.write(i)

        last = i
    stream.write('\n')



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import ast

import numba
from numba import *
from numba import visitors, nodes, error, functions, function_util

logger = logging.getLogger(__name__)

from numba.external import pyapi
from numba.typesystem import is_obj, promote_closest, promote_to_native

#------------------------------------------------------------------------
# Specialize Error Checking and Raising
#------------------------------------------------------------------------

class ExceptionSpecializer(visitors.NumbaTransformer):
    """
    Specialize exception handling. Handle error checking and raising.
    """

    def __init__(self, *args, **kwargs):
        super(ExceptionSpecializer, self).__init__(*args, **kwargs)
        self.visited_callnodes = set()

    #------------------------------------------------------------------------
    # Error Checking
    #------------------------------------------------------------------------

    def visit_CheckErrorNode(self, node):
        if node.badval is not None:
            badval = node.badval
            eq = ast.Eq()
        else:
            assert node.goodval is not None
            badval = node.goodval
            eq = ast.NotEq()

        check = nodes.if_else(eq, node.return_value, badval,
                              lhs=node.raise_node, rhs=None)
        return self.visit(check)

    def visit_PyErr_OccurredNode(self, node):
        check_err = nodes.CheckErrorNode(
            nodes.ptrtoint(function_util.external_call(
                self.context,
                self.llvm_module,
                'PyErr_Occurred')),
            goodval=nodes.ptrtoint(nodes.NULL))
        result = nodes.CloneableNode(node.node)
        result = nodes.ExpressionNode(stmts=[result, check_err],
                                      expr=result.clone)
        return self.visit(result)

    #------------------------------------------------------------------------
    # Error Checking for Function Calls
    #------------------------------------------------------------------------

    def visit_NativeCallNode(self, node):
        badval, goodval = node.badval, node.goodval
        if node not in self.visited_callnodes and (badval is not None or
                                                   goodval is not None):
            self.visited_callnodes.add(node)

            result = node.cloneable
            body = nodes.CheckErrorNode(
                result, badval, goodval,
                node.exc_type, node.exc_msg, node.exc_args)

            node = nodes.ExpressionNode(stmts=[body],
                                        expr=result.clone)
            return self.visit(node)
        else:
            self.generic_visit(node)
            return node

    #------------------------------------------------------------------------
    # Check for UnboundLocalError
    #------------------------------------------------------------------------

    def visit_Name(self, node):
        if (is_obj(node.type) and isinstance(node.ctx, ast.Load) and
            getattr(node, 'cf_maybe_null', False)):
            # Check for unbound objects and raise UnboundLocalError if so
            value = nodes.LLVMValueRefNode(Py_uintptr_t, None)
            node.loaded_name = value

            exc_msg = node.variable.name
            if hasattr(node, 'lineno'):
                exc_msg = '%s%s' % (error.format_pos(node), exc_msg)

            check_unbound = nodes.CheckErrorNode(
                value, badval=nodes.const(0, Py_uintptr_t),
                exc_type=UnboundLocalError,
                exc_msg=exc_msg)
            node.check_unbound = self.visit(check_unbound)

        return node

    #------------------------------------------------------------------------
    # Exception Raising
    #------------------------------------------------------------------------

    def _raise_exception(self, body, node):
        if node.exc_type:
            assert node.exc_msg

            if node.exc_args:
                args = [node.exc_type, node.exc_msg] + node.exc_args
                raise_node = function_util.external_call(self.context,
                                                         self.llvm_module,
                                                         'PyErr_Format',
                                                         args=args)
            else:
                args = [node.exc_type, node.exc_msg]
                raise_node = function_util.external_call(self.context,
                                                         self.llvm_module,
                                                         'PyErr_SetString',
                                                         args=args)

            body.append(raise_node)

    def _trap(self, body, node):
        if node.exc_msg and node.print_on_trap:
            pos = error.format_pos(node)
            if node.exception_type:
                exc_type = '%s: ' % node.exception_type.__name__
            else:
                exc_type = ''

            msg = '%s%s%%s' % (exc_type, pos)
            format = nodes.const(msg, c_string_type)
            print_msg = function_util.external_call(self.context,
                                                    self.llvm_module,
                                                    'printf',
                                                    args=[format,
                                                          node.exc_msg])
            body.append(print_msg)

        trap = nodes.LLVMIntrinsicNode(signature=void(), args=[],
                                       func_name='TRAP')
        body.append(trap)

    def visit_RaiseNode(self, node):
        body = []
        if self.nopython:
            self._trap(body, node)
        else:
            self._raise_exception(body, node)

        body.append(nodes.PropagateNode())
        return ast.Suite(body=body)



# -*- coding: utf-8 -*-
"""
Generate LLVM code for a minivect AST.
"""
from __future__ import print_function, division, absolute_import

import sys

try:
    import llvm.core
    import llvm.ee
    import llvm.passes
except ImportError:
    llvm = None

from . import codegen
from . import minierror
from . import minitypes
from . import minivisitor
from . import ctypes_conversion

def handle_struct_passing(builder, alloca_func, largs, signature):
    """
    Handle signatures with structs. If signature.struct_by_reference
    is set, we need to pass in structs by reference, and retrieve
    struct return values by refererence through an additional argument.

    Structs are always loaded as pointers.
    Complex numbers are always immediate struct values.
    """
    for i, (arg_type, larg) in enumerate(zip(signature.args, largs)):
        if minitypes.pass_by_ref(arg_type):
            if signature.struct_by_reference:
                if arg_type.is_complex:
                    new_arg = alloca_func(arg_type)
                    builder.store(larg, new_arg)
                    larg = new_arg

            largs[i] = larg

    if (signature.struct_by_reference and
            minitypes.pass_by_ref(signature.return_type)):
        return_value = alloca_func(signature.return_type)
        largs.append(return_value)
        return return_value

    return None

class LLVMCodeGen(codegen.CodeGen):
    """
    Generate LLVM code for a minivect AST.

    Takes a regular :py:class:`minivect.minicode.CodeWriter` to which it
    writes the LLVM function and a ctypes function.
    """

    in_lhs_expr = 0

    def __init__(self, context, codewriter):
        super(LLVMCodeGen, self).__init__(context, codewriter)
        self.declared_temps = set()
        self.temp_names = set()

        self.astbuilder = context.astbuilder
        self.blocks = []
        self.symtab = {}
        self.llvm_temps = {}

        import llvm # raise an error at this point if llvm-py is not installed

        self.init_binops()
        self.init_comparisons()

        # List of LLVM call instructions to inline
        self.inline_calls = []

    def append_basic_block(self, name='unamed'):
        "append a basic block and keep track of it"
        idx = len(self.blocks)
        bb = self.lfunc.append_basic_block('%s_%d' % (name, idx))
        self.blocks.append(bb)
        return bb

    def optimize(self):
        "Run llvm optimizations on the generated LLVM code (using the old llvm-py)"
        llvm_fpm = llvm.passes.FunctionPassManager.new(self.llvm_module)
        for llvm_pass in self.context.llvm_passes():
            llvm_fpm.add(llvm_pass)

        llvm_fpm.initialize()
        llvm_fpm.run(self.lfunc)
        llvm_fpm.finalize()

        # self.context.llvm_fpm.run(self.lfunc)

    def optimize(self):
        "Run llvm optimizations on the generated LLVM code"
        for call_instr in self.inline_calls:
            # print 'inlining...', call_instr
            llvm.core.inline_function(call_instr)

        llvm_fpm = llvm.passes.FunctionPassManager.new(self.llvm_module)
        # target_data = llvm.ee.TargetData(self.context.llvm_ee)
        #llvm_fpm.add(self.context.llvm_ee.target_data.clone())
        pmb = llvm.passes.PassManagerBuilder.new()
        pmb.opt_level = 3
        pmb.vectorize = True

        pmb.populate(llvm_fpm)
        llvm_fpm.run(self.lfunc)

    def visit_FunctionNode(self, node):
        self.specializer = node.specializer
        self.function = node
        self.llvm_module = self.context.llvm_module

        name = node.name + node.specialization_name
        node.mangled_name = name

        lfunc_type = node.type.to_llvm(self.context)
        self.lfunc = self.llvm_module.add_function(lfunc_type, node.mangled_name)

        self.entry_bb = self.append_basic_block('entry')
        self.builder = llvm.core.Builder.new(self.entry_bb)

        self.add_arguments(node)
        self.visit(node.body)

        # print self.lfunc
        self.lfunc.verify()
        self.optimize()
        # print self.lfunc

        self.code.write(self.lfunc)

        from numba.codegen.llvmcontext import LLVMContextManager
        ctypes_func = ctypes_conversion.get_ctypes_func(
                    node, self.lfunc, LLVMContextManager().execution_engine,
                                                        self.context)
        self.code.write(ctypes_func)

    def add_arguments(self, function):
        "Insert function arguments into the symtab"
        i = 0
        for arg in function.arguments + function.scalar_arguments:
            if arg.used:
                for var in arg.variables:
                    llvm_arg = self.lfunc.args[i]
                    self.symtab[var.name] = llvm_arg
                    if var.type.is_pointer:
                        llvm_arg.add_attribute(llvm.core.ATTR_NO_ALIAS)
                        llvm_arg.add_attribute(llvm.core.ATTR_NO_CAPTURE)
                    i += 1
            else:
                for var in arg.variables:
                    self.symtab[var.name] = self.visit(var)

    def visit_PrintNode(self, node):
        pass

    def visit_Node(self, node):
        self.visitchildren(node)
        return node

    def visit_OpenMPConditionalNode(self, node):
        "OpenMP is not yet implemented, only process the 'else' directives."
        if node.else_body:
            self.visit(node.else_body)
        return node

    def visit_ForNode(self, node):
        '''
        Implements simple for loops with iternode as range, xrange
        '''
        bb_cond = self.append_basic_block('for.cond')
        bb_incr = self.append_basic_block('for.incr')
        bb_body = self.append_basic_block('for.body')
        bb_exit = self.append_basic_block('for.exit')

        # generate initializer
        self.visit(node.init)
        self.builder.branch(bb_cond)

        # generate condition
        self.builder.position_at_end(bb_cond)
        cond = self.visit(node.condition)
        self.builder.cbranch(cond, bb_body, bb_exit)

        # generate increment
        self.builder.position_at_end(bb_incr)
        self.visit(node.step)
        self.builder.branch(bb_cond)

        # generate body
        self.builder.position_at_end(bb_body)
        self.visit(node.body)
        self.builder.branch(bb_incr)

        # move to exit block
        self.builder.position_at_end(bb_exit)

    def visit_IfNode(self, node):
        cond = self.visit(node.cond)

        bb_true = self.append_basic_block('if.true')
        bb_endif = self.append_basic_block('if.end')

        if node.else_body:
            bb_false = self.append_basic_block('if.false')
        else:
            bb_false = bb_endif

        test = self.visit(node.cond)
        self.builder.cbranch(test, bb_true, bb_false)

        # if cond
        self.builder.position_at_end(bb_true)
        self.visit(node.body)
        self.builder.branch(bb_endif)

        if node.else_body:
            # else
            self.builder.position_at_end(bb_false)
            self.visit(node.else_body)
            self.builder.branch(bb_endif)

        # endif
        self.builder.position_at_end(bb_endif)

    def visit_ReturnNode(self, node):
        self.builder.ret(self.visit(node.operand))

    def visit_CastNode(self, node):
        if node.type.is_pointer:
            result = self.visit(node.operand)
            dest_type = node.type.to_llvm(self.context)
            # print result, dest_type
            # node.print_tree(self.context)
            return self.builder.bitcast(result, dest_type)
            # return result.bitcast(node.type)

        return self.visit_PromotionNode(node)

    def visit_PromotionNode(self, node):
        """
        Handle promotions as inserted by
        :py:class:`minivect.type_promoter.TypePromoter`
        """
        result = self.visit(node.operand)
        type = node.type
        op_type = node.operand.type

        smaller = type.itemsize < op_type.itemsize
        if type.is_int and op_type.is_int:
            op = (('zext', 'sext'), ('trunc', 'trunc'))[smaller][type.signed]
        elif type.is_float and op_type.is_float:
            op =  ('fpext', 'fptrunc')[smaller]
        elif type.is_int and op_type.is_float:
            op = ('fptoui', 'fptosi')[type.signed]
        elif type.is_float and op_type.is_int:
            op = ('fptoui', 'fptosi')[type.signed]
        else:
            raise NotImplementedError((type, op_type))

        ltype = type.to_llvm(self.context)
        return getattr(self.builder, op)(result, ltype)

    def init_binops(self):
        # (float_op, unsigned_int_op, signed_int_op)
        self._binops = {
            '+': ('fadd', 'add', 'add'),
            '-': ('fsub', 'sub', 'sub'),
            '*': ('fmul', 'mul', 'mul'),
            '/': ('fdiv', 'udiv', 'sdiv'),
            '%': ('frem', 'urem', 'srem'),

            '&': (None, 'and_', 'and_'),
            '|': (None, 'or_', 'or_'),
            '^': (None, 'xor', 'xor'),

            # TODO: other ops
        }

    def init_comparisons(self):
        """
        Define binary operation LLVM instructions. Do this in a function in
        case llvm-py is not installed.
        """
        self._compare_mapping_float = {
            '>':  llvm.core.FCMP_OGT,
            '<':  llvm.core.FCMP_OLT,
            '==': llvm.core.FCMP_OEQ,
            '>=': llvm.core.FCMP_OGE,
            '<=': llvm.core.FCMP_OLE,
            '!=': llvm.core.FCMP_ONE,
        }

        self._compare_mapping_sint = {
            '>':  llvm.core.ICMP_SGT,
            '<':  llvm.core.ICMP_SLT,
            '==': llvm.core.ICMP_EQ,
            '>=': llvm.core.ICMP_SGE,
            '<=': llvm.core.ICMP_SLE,
            '!=': llvm.core.ICMP_NE,
        }

        self._compare_mapping_uint = {
            '>':  llvm.core.ICMP_UGT,
            '<':  llvm.core.ICMP_ULT,
            '==': llvm.core.ICMP_EQ,
            '>=': llvm.core.ICMP_UGE,
            '<=': llvm.core.ICMP_ULE,
            '!=': llvm.core.ICMP_NE,
        }

    def visit_BinopNode(self, node):
        lhs = self.visit(node.lhs)
        rhs = self.visit(node.rhs)

        op = node.operator
        if (node.type.is_int or node.type.is_float) and node.operator in self._binops:
            idx = 0 if node.type.is_float else node.type.is_int + node.type.signed
            llvm_method_name = self._binops[op][idx]
            meth = getattr(self.builder, llvm_method_name)
            if not lhs.type == rhs.type:
                node.print_tree(self.context)
                assert False, (node.lhs.type, node.rhs.type, lhs.type, rhs.type)
            return meth(lhs, rhs)
        elif node.operator in self._compare_mapping_float:
            return self.generate_compare(node, op, lhs, rhs)
        elif node.type.is_pointer:
            if node.rhs.type.is_pointer:
                lhs, rhs = rhs, lhs
            return self.builder.gep(lhs, [rhs])
        else:
            raise minierror.CompileError(
                node, "Binop %s (type=%s) not implemented for types (%s, %s)" % (
                                                op, node.type, lhs.type, rhs.type))

    def generate_compare(self, node, op, lhs_value, rhs_value):
        op = node.operator
        lop = None

        if node.lhs.type.is_float and node.rhs.type.is_float:
            lfunc = self.builder.fcmp
            lop = self._compare_mapping_float[op]
        elif node.lhs.type.is_int and node.rhs.type.is_int:
            lfunc = self.builder.icmp
            if node.lhs.type.signed and node.rhs.type.signed:
                lop = self._compare_mapping_sint[op]
            elif not (node.lhs.type.signed or node.rhs.type.signed):
                lop = self._compare_mapping_uint[op]

        if lop is None:
            raise minierror.CompileError(
                node, "%s for types (%s, %s)" % (node.operator,
                                                 node.lhs.type, node.rhs.type))

        return lfunc(lop, lhs_value, rhs_value)

    def visit_UnopNode(self, node):
        result = self.visit(node.operand)
        if node.operator == '-':
            return self.builder.neg(result)
        elif node.operator == '+':
            return result
        else:
            raise NotImplementedError(node.operator)

    def visit_TempNode(self, node):
        if node not in self.declared_temps:
            llvm_temp = self._declare_temp(node)
        else:
            llvm_temp = self.llvm_temps[node]

        if self.in_lhs_expr:
            return llvm_temp
        else:
            return self.builder.load(llvm_temp)

    def _mangle_temp(self, node):
        name = node.repr_name or node.name
        if name in self.temp_names:
            name = "%s%d" % (name, len(self.declared_temps))
        node.name = name
        self.temp_names.add(name)
        self.declared_temps.add(node)

    def _declare_temp(self, node, rhs_result=None):
        if node not in self.declared_temps:
            self._mangle_temp(node)

        llvm_temp = self.alloca(node.type)
        self.llvm_temps[node] = llvm_temp
        return llvm_temp

    def alloca(self, type, name=''):
        bb = self.builder.basic_block
        self.builder.position_at_beginning(self.entry_bb)
        llvm_temp = self.builder.alloca(type.to_llvm(self.context), name)
        self.builder.position_at_end(bb)
        return llvm_temp

    def visit_AssignmentExpr(self, node):
        self.in_lhs_expr += 1
        lhs = self.visit(node.lhs)
        self.in_lhs_expr -= 1
        rhs = self.visit(node.rhs)
        return self.builder.store(rhs, lhs)

    def visit_SingleIndexNode(self, node):
        in_lhs_expr = self.in_lhs_expr
        if in_lhs_expr:
            self.in_lhs_expr -= 1
        lhs = self.visit(node.lhs)
        rhs = self.visit(node.rhs)
        if in_lhs_expr:
            self.in_lhs_expr += 1

        result = self.builder.gep(lhs, [rhs])

        if self.in_lhs_expr:
            return result
        else:
            return self.builder.load(result)

    def visit_DereferenceNode(self, node):
        node = self.astbuilder.index(node.operand, self.astbuilder.constant(0))
        return self.visit_SingleIndexNode(node)

    def visit_SizeofNode(self, node):
        return self.visit(self.astbuilder.constant(node.sizeof_type.itemsize,
                                                   node.type))

    def visit_Variable(self, node):
        value = self.symtab[node.name]
        return value

    def visit_ArrayAttribute(self, node):
        return self.symtab[node.name]

    def visit_NoopExpr(self, node):
        pass

    def visit_ResolvedVariable(self, node):
        return self.visit(node.element)

    def visit_JumpNode(self, node):
        return self.builder.branch(self.visit(node.label))

    def visit_JumpTargetNode(self, node):
        basic_block = self.visit(node.label)
        self.builder.branch(basic_block)
        self.builder.position_at_end(basic_block)

    def visit_LabelNode(self, node):
        if node not in self.labels:
            self.labels[node] = self.append_basic_block(node.label)

        return self.labels[node]

    def handle_string_constant(self, b, constant):
        # Create a global string constant, if it doesn't already
        # Based on code in Numba. Seems easier than creating a stack variable
        string_constants = self.context.string_constants = getattr(
                                    self.context, 'string_constants', {})
        if constant in string_constants:
            lvalue = string_constants[constant]
        else:
            lstring = llvm.core.Constant.stringz(constant)
            lvalue = self.context.llvm_module.add_global_variable(
                        lstring.type, "__string_%d" % len(string_constants))
            lvalue.initializer = lstring
            lvalue.linkage = llvm.core.LINKAGE_INTERNAL

            lzero = self.visit(b.constant(0))
            lvalue = self.builder.gep(lvalue, [lzero, lzero])
            string_constants[constant] = lvalue

        return lvalue

    def visit_ConstantNode(self, node):
        b = self.astbuilder

        ltype = node.type.to_llvm(self.context)
        constant = node.value

        if node.type.is_float:
            lvalue = llvm.core.Constant.real(ltype, constant)
        elif node.type.is_int:
            lvalue = llvm.core.Constant.int(ltype, constant)
        elif node.type.is_pointer and self.pyval == 0:
            lvalue = llvm.core.ConstantPointerNull
        elif node.type.is_c_string:
            lvalue = self.handle_string_constant(b, constant)
        else:
            raise NotImplementedError("Constant %s of type %s" % (constant,
                                                                  node.type))

        return lvalue

    def visit_FuncCallNode(self, node):
        llvm_args = list(self.results(node.args))
        llvm_func = self.visit(node.func_or_pointer)

        signature = node.func_or_pointer.type
        if signature.struct_by_reference:
            result = handle_struct_passing(
                        self.builder, self.alloca, llvm_args, signature)

        llvm_call = self.builder.call(llvm_func, llvm_args)

        if node.inline:
            self.inline_calls.append(llvm_call)

        if (signature.struct_by_reference and
                minitypes.pass_by_ref(signature.return_type)):
            return self.builder.load(result)
        else:
            return llvm_call

    def visit_FuncNameNode(self, node):
        func_type = node.type.to_llvm(self.context)
        func = self.context.llvm_module.get_or_insert_function(func_type,
                                                               node.name)
        return func

    def visit_FuncRefNode(self, node):
        raise NotImplementedError



from numba.testing.test_support import parametrize

@parametrize('foo', 'bar')
def func(arg):
    return arg

assert func_testcase.__name__ == 'func'

assert hasattr(func_testcase, 'func_0')
assert hasattr(func_testcase, 'func_1')

assert func_testcase('func_0').func_0() == 'foo'
assert func_testcase('func_1').func_1() == 'bar'



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *
import numba.nodes

#------------------------------------------------------------------------
# Utilities
#------------------------------------------------------------------------

def get_pointer_address(value, type):
    if type.is_known_pointer:
        return type.address
    else:
        return value

def is_null_constant(constant):
    return constant is _NULL

#------------------------------------------------------------------------
# Constant Nodes
#------------------------------------------------------------------------

class ConstNode(ExprNode):
    """
    Wrap a constant.
    """

    _attributes = ['type', 'pyval']

    def __init__(self, pyval, type=None):
        if type is None:
            type = context.typemapper.from_python(pyval)

        # if pyval is not _NULL:
        #     assert not type.is_object

        self.variable = Variable(type, is_constant=True, constant_value=pyval)
        self.type = type
        self.pyval = pyval

    def cast(self, dst_type):
        # This should probably happen in a transform !
        if dst_type.is_int:
            caster = int
        elif dst_type.is_float:
            caster = float
        elif dst_type.is_complex:
            caster = complex
        else:
            raise NotImplementedError(dst_type)

        try:
            self.pyval = caster(self.pyval)
        except ValueError:
            if dst_type.is_int and self.type.is_c_string:
                raise
            raise minierror.UnpromotableTypeError((dst_type, self.type))

        self.type = dst_type
        self.variable.type = dst_type

    def __repr__(self):
        return "const(%s, %s)" % (self.pyval, self.type)

#------------------------------------------------------------------------
# NULL Constants
#------------------------------------------------------------------------

_NULL = object()
NULL_obj = ConstNode(_NULL, object_)
NULL = ConstNode(_NULL, void.pointer())



# -*- coding: utf-8 -*-
'''
Implements basic vectorize
'''
from __future__ import print_function, division, absolute_import
from llvm.core import *
from llvm_cbuilder import *
import llvm_cbuilder.shortnames as C
import numpy as np

from . import _common

class BasicUFunc(CDefinition):
    '''a generic ufunc that wraps the workload
    '''
    _argtys_ = [
        ('args',       C.pointer(C.char_p), [ATTR_NO_ALIAS]),
        ('dimensions', C.pointer(C.intp), [ATTR_NO_ALIAS]),
        ('steps',      C.pointer(C.intp), [ATTR_NO_ALIAS]),
        ('data',       C.void_p, [ATTR_NO_ALIAS]),
    ]

    def body(self, args, dimensions, steps, data):
        ufunc_ptr = self.depends(self.FuncDef)
        fnty = ufunc_ptr.type.pointee

        arg_ptrs = []
        arg_steps = []
        for i in range(len(fnty.args)+1):
            arg_ptrs.append(self.var_copy(args[i]))
            const_steps = self.var_copy(steps[i])
            const_steps.invariant = True
            arg_steps.append(const_steps)

        with self.for_range(dimensions[0]) as (loop, item):
            callargs = []
            for i, argty in enumerate(fnty.args):
                casted = arg_ptrs[i].cast(C.pointer(argty))
                callargs.append(casted.load())
                arg_ptrs[i].assign(arg_ptrs[i][arg_steps[i]:]) # increment pointer

            res = ufunc_ptr(*callargs, **dict(inline=True))
            retval_ptr = arg_ptrs[-1].cast(C.pointer(fnty.return_type))
            retval_ptr.store(res, nontemporal=True)
            arg_ptrs[-1].assign(arg_ptrs[-1][arg_steps[-1]:])

        self.ret()

    def specialize(cls, func_def):
        '''specialize to a workload
        '''
        cls._name_ = 'basicufunc_%s'% (func_def)
        cls.FuncDef = func_def

class _BasicVectorizeFromFunc(_common.CommonVectorizeFromFunc):
    def build(self, lfunc, dtypes):
        def_buf = BasicUFunc(CFuncRef(lfunc))
        func = def_buf(lfunc.module)
        _common.post_vectorize_optimize(func)
        return func

basic_vectorize_from_func = _BasicVectorizeFromFunc()

class BasicASTVectorize(_common.GenericASTVectorize):

    _from_func_factory = basic_vectorize_from_func

    def build_ufunc(self, dispatcher=None):
        return self._from_func(dispatcher=dispatcher)

BasicVectorize = BasicASTVectorize



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import llvm.core
from numba.minivect import minitypes

from collections import namedtuple

Signature = namedtuple('Signature', ['return_type', 'arg_types'])

class Intrinsic(object):
    _attributes = ('func_name', 'arg_types', 'return_type')
    func_name = None
    arg_types = None
    return_type = None
    linkage = llvm.core.LINKAGE_LINKONCE_ODR
    is_vararg = False

    # Unused?
    #    badval = None
    #    goodval = None
    #    exc_type = None
    #    exc_msg = None
    #    exc_args = None

    def __init__(self, **kwargs):
        if __debug__:
            # Only accept keyword arguments defined _attributes
            for k, v in kwargs.items():
                if k not in self._attributes:
                    raise TypeError("Invalid keyword arg %s -> %s" % (k, v))
        vars(self).update(kwargs)

    @property
    def signature(self):
        return minitypes.FunctionType(return_type=self.return_type,
                                      args=self.arg_types,
                                      is_vararg=self.is_vararg)

    @property
    def name(self):
        if self.func_name is None:
            return type(self).__name__
        else:
            return self.func_name

    def implementation(self, module, lfunc):
        return None


class IntrinsicLibrary(object):
    '''An intrinsic library maintains a LLVM module for holding the
        intrinsics.  These are functions are used internally to implement
        specific features.
        '''
    def __init__(self, context):
        self._context = context
        self._module = llvm.core.Module.new('intrlib.%X' % id(self))
        # A lookup dictionary that matches (name) -> (intr)
        self._functions = {}
        # (name, args) -> (lfunc)
        self._compiled = {}
        # Build function pass manager to reduce memory usage of
        from llvm.passes import FunctionPassManager, PassManagerBuilder
        pmb = PassManagerBuilder.new()
        pmb.opt_level = 2
        self._fpm = FunctionPassManager.new(self._module)
        pmb.populate(self._fpm)

    def add(self, intr):
        '''Add a new intrinsic.
        intr --- an Intrinsic class
        '''
        if __debug__:
            # Sentry for duplicated external function name
            if intr.__name__ in self._functions:
                raise NameError("Duplicated intrinsic function: %s" \
                                % intr.__name__)
        self._functions[intr.__name__] = intr
        if intr.arg_types and intr.return_type:
            # only if it defines arg_types and return_type
            self.implement(intr())

    def implement(self, intr):
        '''Implement a new intrinsic.
        intr --- an Intrinsic class
        '''
        # implement the intrinsic
        lfunc_type = intr.signature.to_llvm(self._context)
        lfunc = self._module.add_function(lfunc_type, name=intr.name)
        lfunc.linkage = intr.linkage
        intr.implementation(lfunc.module, lfunc)

        # optimize the function
        self._fpm.run(lfunc)

        # populate the lookup dictionary
        key = type(intr).__name__, tuple(intr.arg_types)
        sig = intr.signature.to_llvm(self._context),
        self._compiled[key] = sig, lfunc

    def declare(self, module, name, arg_types=(), return_type=None):
        '''Create a declaration in the module.
        '''
        sig, intrlfunc = self.get(name, arg_types, return_type)
        lfunc = module.get_or_insert_function(intrlfunc.type.pointee,
                                              name=intrlfunc.name)
        return sig, lfunc
    
    def get(self, name, arg_types=(), return_type=None):
        '''Get an intrinsic by name and sig

        name --- function name
        arg_types --- function arg types
        return_types --- function return type
            
        Returns the function signature and a lfunc pointing to the 
        '''
        if not arg_types and not return_type:
            intr = self._functions[name]
            sig, lfunc = self.get(name,
                                  arg_types=intr.arg_types,
                                  return_type=intr.return_type)
        else:
            key = name, tuple(arg_types)
            try:
                sig, lfunc = self._compiled[key]
            except KeyError:
                intr = self._functions[name]
                if not intr.arg_types and not intr.return_type:
                    self.implement(intr(arg_types=arg_types,
                                        return_type=return_type))
                sig, lfunc = self._compiled[key]

        return sig, lfunc

    def link(self, module):
        '''Link the intrinsic library into the target module.
        '''
        module.link_in(self._module, preserve=True)



# -*- coding: utf-8 -*-

"""
Handle signatures of methods in @jit and @autojit classes.
"""

from __future__ import print_function, division, absolute_import

import types

import numba
from numba import *
from numba import error
from numba import typesystem
from numba.exttypes import types as etypes
from numba.minivect import minitypes

#------------------------------------------------------------------------
# Parse method signatures
#------------------------------------------------------------------------

class Method(object):
    """
    py_func: the python 'def' function
    """

    def __init__(self, py_func, name, signature, is_class, is_static,
                 nopython=False):
        self.py_func = py_func
        # py_func.live_objects = []

        # Name of this function, py_func.__name__ is inherently unreliable
        self.name = name
        self.signature = signature

        self.is_class = is_class
        self.is_static = is_static

        self.nopython = nopython
        self.template_signature = None

        # Filled out after extension method is compiled
        # (ExtensionCompiler.compile_methods())
        self.wrapper_func = None
        self.lfunc = None
        self.lfunc_pointer = None

    def get_wrapper(self):
        if self.is_class:
            return classmethod(self.wrapper_func)
        elif self.is_static:
            return staticmethod(self.wrapper_func)
        else:
            return self.wrapper_func

    def update_from_env(self, func_env):
        self.lfunc = func_env.lfunc
        self.lfunc_pointer = func_env.translator.lfunc_pointer
        self.wrapper_func = func_env.numba_wrapper_func

    def clone(self):
        return type(self)(self.py_func, self.name, self.signature,
                          self.is_class, self.is_static, self.nopython)


#------------------------------------------------------------------------
# Utilities
#------------------------------------------------------------------------

def get_classmethod_func(func):
    """
    Get the Python function the classmethod or staticmethod is wrapping.

    In Python2.6 classmethod and staticmethod don't have the '__func__'
    attribute.
    """
    if isinstance(func, classmethod):
        return func.__get__(object()).__func__
    else:
        assert isinstance(func, staticmethod)
        return func.__get__(object())

#------------------------------------------------------------------------
# Method Builders
#------------------------------------------------------------------------

class MethodMaker(object):
    """
    Creates Methods from python functions and validates user-declared
    signatures.
    """

    def no_signature(self, method):
        "Called when no signature is found for the method"

    def default_signature(self, method, ext_type):
        """
        Retrieve the default method signature for the given method if
        no user-declared signature exists.
        """
        if has_known_signature(method):
            # We know the argument types, but we don't have a solid
            # infrastucture for inter-procedural type inference yet
            # return typesystem.function(None, [])
            return None
        else:
            return None

    def make_method_type(self, method):
        "Create a method type for the given Method and declared signature"
        restype = method.signature.return_type
        argtypes = method.signature.args
        signature = etypes.ExtMethodType(
                    return_type=restype, args=argtypes, name=method.name,
                    is_class=method.is_class, is_static=method.is_static)
        return signature

def has_known_signature(method):
    argcount = method.py_func.__code__.co_argcount
    return ((method.is_static and argcount == 0) or
            (not method.is_static and argcount == 1))

# ______________________________________________________________________
# Method processing for @jit classes

class JitMethodMaker(MethodMaker):

    def no_signature(self, py_func):
        if py_func.__name__ != '__init__':
            raise error.NumbaError(
                "Method '%s' does not have signature" % (py_func.__name__,))

    def default_signature(self, method, ext_type):
        if method.name == '__init__':
            argtypes = [numba.object_] * (method.py_func.__code__.co_argcount - 1)
            default_signature = numba.void(*argtypes)
            return default_signature
        else:
            return super(JitMethodMaker, self).default_signature(
                method, ext_type)

# ______________________________________________________________________
# Method processing for @autojit classes

class AutojitMethodMaker(MethodMaker):

    def __init__(self, argtypes):
        self.argtypes = argtypes

    def default_signature(self, method, ext_type):
        if method.name == '__init__':
            default_signature = numba.void(*self.argtypes)
            return default_signature
        else:
            return super(AutojitMethodMaker, self).default_signature(
                method, ext_type)



#------------------------------------------------------------------------
# Method signature parsing
#------------------------------------------------------------------------

def method_argtypes(method, ext_type, argtypes):
    if method.is_static:
        leading_arg_types = ()
    elif method.is_class:
        leading_arg_types = (numba.object_,)
    else:
        leading_arg_types = (ext_type,)

    return leading_arg_types + tuple(argtypes)

def process_signature(method, method_name, method_maker=MethodMaker()):
    """
    Verify a method signature.

    Returns a Method object and the resolved signature.
    Returns None if the object isn't a method.
    """

    signature = None

    is_static=False
    is_class=False

    while True:
        if isinstance(method, types.FunctionType):
            # Process function
            if signature is None:
                method_maker.no_signature(method)

            method = Method(method, method_name, signature,
                            is_class, is_static)
            return method

        elif isinstance(method, minitypes.Function):
            # @double(...)
            # def func(self, ...): ...
            signature = method.signature
            method = method.py_func

        else:
            # Process staticmethod and classmethod
            if isinstance(method, staticmethod):
                is_static = True
            elif isinstance(method, classmethod):
                is_class = True
            else:
                return None

            method = get_classmethod_func(method)

    assert False # Unreachable

class MethodSignatureProcessor(object):
    """
    Processes signatures of extension types.
    """

    def __init__(self, class_dict, ext_type, method_maker, validators):
        self.class_dict = class_dict
        self.ext_type = ext_type
        self.method_maker = method_maker

        # List of method validators: [MethodValidator]
        self.validators = validators

    def update_signature(self, method):
        """
        Update a method signature with the extension type for 'self'.

            class Foo(object):
                @void()                 # becomes: void(ext_type(Foo))
                def method(self): ...
        """
        argtypes = method_argtypes(method, self.ext_type, method.signature.args)
        restype = method.signature.return_type
        method.signature = typesystem.function(restype, argtypes)

        method.signature = self.method_maker.make_method_type(method)

    def get_method_signatures(self):
        """
        Return [Method] for each decorated method in the class
        """
        methods = []

        for method_name, method in sorted(self.class_dict.iteritems()):
            method = process_signature(method, method_name)
            if method is None:
                continue

            for validator in self.validators:
                validator.validate(method, self.ext_type)

            if method.signature is None:
                method.signature = self.method_maker.default_signature(
                    method, self.ext_type)

            if method.signature is not None:
                self.update_signature(method)

            methods.append(method)

        return methods




# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from functools import partial

import numba as nb
from numba.containers import orderedcontainer

import numpy as np

INITIAL_BUFSIZE = 5

def notimplemented(msg):
    raise NotImplementedError("'%s' method of type 'typedtuple'" % msg)

_tuple_cache = {}

#-----------------------------------------------------------------------
# Runtime Constructor
#-----------------------------------------------------------------------

def typedtuple(item_type, iterable=None, _tuple_cache=_tuple_cache):
    """
    >>> typedtuple(nb.int_)
    ()
    >>> ttuple = typedtuple(nb.int_, range(10))
    >>> ttuple
    (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
    >>> ttuple[5]
    5L

    >>> typedtuple(nb.float_, range(10))
    (0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0)
    """
    typedtuple_ctor = compile_typedtuple(item_type)
    return typedtuple_ctor(iterable)

#-----------------------------------------------------------------------
# Typedlist implementation
#-----------------------------------------------------------------------

def compile_typedtuple(item_type, _tuple_cache=_tuple_cache):
    if item_type in _tuple_cache:
        return _tuple_cache[item_type]

    dtype = item_type.get_dtype()
    methods = orderedcontainer.container_methods(item_type, notimplemented)

    @nb.jit(warn=False)
    class typedtuple(object):

        @nb.void(nb.object_)
        def __init__(self, iterable):

            self.size = 0

            # TODO: Use length hint of iterable for initial buffer size
            self.buf = np.empty(INITIAL_BUFSIZE, dtype=dtype)

            if iterable != None:
                self.__extend(iterable)

        __getitem__ = methods['getitem']
        __append = methods['append']
        index = methods['index']
        count = methods['count']

        @nb.void(nb.object_)
        def __extend(self, iterable):
            for obj in iterable:
                self.__append(obj)

        @nb.Py_ssize_t()
        def __len__(self):
            return self.size

        @nb.c_string_type()
        def __repr__(self):
            buf = ", ".join([str(self.buf[i]) for i in range(self.size)])
            return "(" + buf + ")"

    _tuple_cache[item_type] = typedtuple
    return typedtuple


if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.control_flow.control_flow import (ControlBlock, ControlFlowAnalysis,
                                             FuncDefExprNode)
from numba.control_flow.cfstats import *
from numba.control_flow.delete_cfnode import DeleteStatement


# -*- coding: utf-8 -*-

"""
Test numba.addressof().
"""

from __future__ import print_function, division, absolute_import

import ctypes

import numba
from numba import *


@jit(int32(int32, int32))
def func(a, b):
    return a * b

@autojit
def error_func():
    pass

# TODO: struct pointer support

before_computed_column = struct([
    ('x', float32),
    ('y', float32)])

with_computed_column = struct([
    ('mean', float32),
    ('x', float32),
    ('y', float32)])

signature = void(with_computed_column.ref(),
                 before_computed_column.ref())

# @jit(signature, nopython=True)
def cc_kernel(dst, src):
    dst.mean = (src.x + src.y) / 2.0
    dst.x = src.x
    dst.y = src.y

#------------------------------------------------------------------------
# Tests
#------------------------------------------------------------------------

def test_addressof(arg):
    """
    >>> func = test_addressof(func)
    >>> assert func.restype == ctypes.c_int32
    >>> assert func.argtypes == (ctypes.c_int32, ctypes.c_int32)
    >>> func(5, 2)
    10
    """
    return numba.addressof(arg)

def test_addressof_error(arg, **kwds):
    """
    >>> test_addressof_error(error_func)
    Traceback (most recent call last):
        ...
    TypeError: Object is not a jit function

    >>> test_addressof_error(func, propagate=False)
    Traceback (most recent call last):
        ...
    ValueError: Writing unraisable exception is not yet supported
    """
    return numba.addressof(arg, **kwds)

def test_address_of_struct_function():
    S1 = before_computed_column.to_ctypes()
    S2 = with_computed_column.to_ctypes()
    ctypes_kernel = numba.addressof(cc_kernel)

    s1 = S1(10, 5)
    s2 = S2(0, 0, 0)
    ctypes_kernel(s2, s1)

    assert s2.x == s1.x
    assert s2.y == s1.y
    assert s2.mean == (s1.x + s1.y) / 2.0


numba.testing.testmod()



# -*- coding: utf-8 -*-
"""
Visitor to generate a Graphviz .dot file with an AST representation.
"""
from __future__ import print_function, division, absolute_import

from .pydot import pydot

from . import minivisitor

class GraphvizGenerator(minivisitor.PrintTree):
    """
    Render a minivect AST as a graphviz tree.
    """

    def __init__(self, context, name, node_color=None, edge_color=None,
                 node_fontcolor=None, edge_fontcolor=None):
        super(GraphvizGenerator, self).__init__(context)
        self.name = name
        self.counter = 0
        self.node_color = node_color
        self.edge_color = edge_color
        self.node_fontcolor = node_fontcolor
        self.edge_fontcolor = edge_fontcolor

    def create_node(self, node):
        "Create a graphviz node from the miniast node"
        label = '"%s"' % self.format_node(node, want_type_info=False)
        self.counter += 1
        pydot_node = pydot.Node(str(self.counter), label=label)
        self.add_node(pydot_node)
        return pydot_node

    def add_node(self, pydot_node):
        "Add a pydot node to the graph and set its colors"
        if self.node_color is not None:
            pydot_node.set_color(self.node_color)
        if self.node_fontcolor is not None:
            pydot_node.set_fontcolor(self.node_fontcolor)

        self.graph.add_node(pydot_node)

    def add_edge(self, source, dest, edge_label=None):
        "Add an edge between two pydot nodes and set the colors"
        edge = pydot.Edge(source, dest)

        if edge_label is not None:
            edge.set_label(edge_label)
        if self.edge_color is not None:
            edge.set_color(self.edge_color)
        if self.edge_fontcolor is not None:
            edge.set_fontcolor(self.edge_fontcolor)

        self.graph.add_edge(edge)

    def visit_Node(self, node, pydot_node=None):
        "Visit children and add edges to their Graphviz nodes."
        if pydot_node is None:
            pydot_node = self.create_node(node)

        nodes_dict = self.visitchildren(node)
        attrs = self.context.getchildren(node)

        for attr in attrs:
            values = nodes_dict.get(attr, None)
            if values is not None:
                if isinstance(values, list):
                    for value in values:
                        self.add_edge(pydot_node, value)
                else:
                    self.add_edge(pydot_node, values, attr)

        return pydot_node

    def visit_FunctionNode(self, node):
        "Create a graphviz graph"
        self.graph = pydot.Dot(self.name, graph_type='digraph')

        pydot_function = self.create_node(node)
        pydot_body = self.visit(node.body)

        # Create artificial arguments for brevity
        pydot_args = pydot.Node("Arguments (omitted)")
        self.add_node(pydot_args)

        self.add_edge(pydot_function, pydot_body)
        self.add_edge(pydot_function, pydot_args)

        return self.graph



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

# This tests a potential GCC 4.1.2 miscompile of LLVM.
# The problem is observed as a error in greedy register allocation pass,
# which resulted as a segfault.
# No such problem in GCC 4.4.6.


from numba import *
import numpy as np

@jit(uint8[:,:](f8, f8, f8, f8, uint8[:,:], int32))
def create_fractal(min_x, max_x, min_y, max_y, image, iters):
    return image






"""
>>> boom()
Traceback (most recent call last):
    ...
ValueError: invalid literal for int() with base 10: 'boom'
>>> boom2()
Traceback (most recent call last):
    ...
TypeError: 'object' object is not callable
>>> boom3()
Traceback (most recent call last):
    ...
TypeError: 'object' object is not callable
"""

import sys
import ctypes

from numba import *
import numpy as np

@autojit(backend='ast')
def boom():
    return int('boom')

@jit(int_())
def boom2():
    return object()('boom')

@jit(complex128())
def boom3():
    return object()('boom')

if __name__ == "__main__":
    import numba
    numba.testing.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.nodes import *

# not really an expression, but used in an assignment
class ClosureNode(ExprNode):
    """
    Inner functions or closures.

    When coerced to an object, a wrapper PyMethodDef gets created, and at
    call time a function is dynamically created with the closure scope.

        func_def:
            AST FunctionDef of the function
        closure_type:
            numba.typesystem.ClosureType
        outer_py_func:
            Outer Python function (or None!)
    """

    _fields = []

    def __init__(self, env, func_def, closure_type, outer_py_func, **kwargs):
        super(ClosureNode, self).__init__(**kwargs)
        self.func_def = func_def
        self.type = closure_type
        self.outer_py_func = outer_py_func
        self.name = self.func_def.name

        func_env = env.translation.get_env(func_def)
        self.need_numba_func = not func_env or func_env.need_closure_wrapper
        self.lfunc = None
        self.wrapper_func = None
        self.wrapper_lfunc = None
        self.lfunc_pointer = None

        # FunctionEnvironment after type inference
        self.func_env = None
        # self.type_inferred_ast = None
        # self.symtab = None

        from numba import pipeline
        self.locals = pipeline.get_locals(func_def, None)

        # The Python extension type that must be instantiated to hold cellvars
        # self.scope_type = None
        self.ext_type = None
        self.need_closure_scope = False

    def make_pyfunc(self):
        d = self.outer_py_func.__globals__
#        argnames = tuple(arg.id for arg in self.func_def.args.args)
#        dummy_func_string = """
#def __numba_closure_func(%s):
#    pass
#        """ % ", ".join(argnames)
#        exec dummy_func_string in d, d

        # Something set a pure and original, unmodified, AST, use that
        # instead and reset it after the compile. This is a HACK
        func_body = self.func_def.body
        if hasattr(self.func_def, 'pure_ast_body'):
            self.func_def.body = self.func_def.pure_ast_body

        name = self.func_def.name
        self.func_def.name = '__numba_closure_func'
        ast_mod = ast.Module(body=[self.func_def])
        numba.functions.fix_ast_lineno(ast_mod)
        c = compile(ast_mod, '<string>', 'exec')
        exec(c, d, d)
        self.func_def.name = name

        self.py_func = d['__numba_closure_func']
        self.py_func.live_objects = []
        self.py_func.__module__ = self.outer_py_func.__module__
        self.py_func.__name__ = name

        if hasattr(self.func_def, 'pure_ast_body'):
            self.func_def.body = func_body

class InstantiateClosureScope(ExprNode):

    _fields = ['outer_scope']

    def __init__(self, func_def, scope_ext_type, scope_type, outer_scope, **kwargs):
        super(InstantiateClosureScope, self).__init__(**kwargs)
        self.func_def = func_def
        self.scope_type = scope_type
        self.ext_type = scope_ext_type
        self.outer_scope = outer_scope
        self.type = scope_type

class ClosureScopeLoadNode(ExprNode):
    "Load the closure scope for the function or NULL"

    type = void.pointer()

class ClosureCallNode(NativeCallNode):
    """
    Call to closure or inner function.
    """

    _fields = ['func', 'args']

    def __init__(self, closure_type, call_node, **kwargs):
        self.call_node = call_node
        self.func = call_node.func
        self.closure_type = closure_type

        self.argnames = [name.id for name in self.func_def.args.args[self.need_closure_scope:]]
        self.expected_nargs = len(self.argnames)

        args, keywords = call_node.args, call_node.keywords
        args = args + self._resolve_keywords(args, keywords)
        super(ClosureCallNode, self).__init__(
                closure_type.signature, args, llvm_func=None,
                skip_self=self.need_closure_scope, **kwargs)

    @property
    def need_closure_scope(self):
        return self.closure_type.closure.need_closure_scope

    @property
    def func_def(self):
        return self.closure_type.closure.func_def

    def _resolve_keywords(self, args, keywords):
        "Map keyword arguments to positional arguments"
        expected = self.expected_nargs - len(args)
        if len(keywords) != expected:
            raise error.NumbaError(
                    self.call_node,
                    "Expected %d arguments, got %d" % (self.expected_nargs,
                                                       len(args) + len(keywords)))

        argpositions = dict(zip(self.argnames, range(self.expected_nargs)))
        positional = [None] * (self.expected_nargs - len(args))

        for keyword in keywords:
            argname = keyword.arg
            pos = argpositions.get(argname, None)
            if pos is None:
                raise error.NumbaError(
                        keyword, "Not a valid keyword argument name: %s" % argname)
            elif pos < len(args):
                raise error.NumbaError(
                        keyword, "Got multiple values for positional "
                                 "argument %r" % argname)
            else:
                positional[pos] = keyword.value

        return positional



import provider
import consumer

class ProviderSubclassOnHeap(provider.Provider):
    pass

print consumer.sum_baseline(provider.Provider(), 4)
print consumer.sum_baseline(object(), 4)
print consumer.sum_baseline(ProviderSubclassOnHeap(), 4)




# -*- coding: utf-8 -*-
# Thanks to Gaëtan de Menten

"""
>>> test_valid_compare()
>>> invalid_compare(np.arange(10))
Traceback (most recent call last):
      ...
NumbaError: 27:11: Cannot determine truth value of boolean array (use any or all)
"""

import numpy as np
from numba import autojit, jit, double, b1

def array(a):
    return a > 0.1

#fails too
#array_nb = jit(b1[:](double[:]))(array)
def test_valid_compare():
    array_nb = autojit(array)
    a = np.random.rand(1e6)
    assert np.allclose(array(a), array_nb(a))

@autojit
def invalid_compare(a):
    return 1 < a < 2


if __name__ == '__main__':
    from numba.testing import test_support
    test_support.testmod()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from .external import ExternalFunction
from numba import *

class ofunc(ExternalFunction):
    arg_types = [object_]
    return_type = object_

class Py_IncRef(ofunc):
    # TODO: rewrite calls to Py_IncRef/Py_DecRef to direct integer
    # TODO: increments/decrements
    return_type = void

class Py_DecRef(Py_IncRef):
    pass

class PyObject_Length(ofunc):
    return_type = Py_ssize_t

class PyObject_Call(ExternalFunction):
    arg_types = [object_, object_, object_]
    return_type = object_

class PyObject_CallMethod(ExternalFunction):
    arg_types = [object_, c_string_type, c_string_type]
    return_type = object_
    is_vararg = True

class PyObject_CallMethodObjArgs(ExternalFunction):
    arg_types = [object_, c_string_type]
    return_type = object_
    is_vararg = True

class PyObject_Type(ExternalFunction):
    '''
        Added to aid debugging
        '''
    arg_types = [object_]
    return_type = object_

class PyTuple_Pack(ExternalFunction):
    arg_types = [Py_ssize_t]
    return_type = object_
    is_vararg = True

class Py_BuildValue(ExternalFunction):
    arg_types = [c_string_type]
    return_type = object_
    is_vararg = True

class PyArg_ParseTuple(ExternalFunction):
    arg_types = [object_, c_string_type]
    return_type = int_
    is_vararg = True

class PyObject_Print(ExternalFunction):
    arg_types = [object_, void.pointer(), int_]
    return_type = int_

class PyObject_Str(ExternalFunction):
    arg_types = [object_]
    return_type = object_

class PyObject_GetAttrString(ExternalFunction):
    arg_types = [object_, c_string_type]
    return_type = object_

class PyObject_GetItem(ExternalFunction):
    arg_types = [object_, object_]
    return_type = object_

class PyObject_SetItem(ExternalFunction):
    arg_types = [object_, object_, object_]
    return_type = int_

class PyObject_GetIter(ExternalFunction):
    arg_types = [object_]
    return_type = object_

class PyIter_Next(ExternalFunction):
    arg_types = [object_]
    return_type = object_

class PySlice_New(ExternalFunction):
    arg_types = [object_, object_, object_]
    return_type = object_

class PyErr_SetString(ExternalFunction):
    arg_types = [object_, c_string_type]
    return_type = void

class PyErr_Format(ExternalFunction):
    arg_types = [object_, c_string_type]
    return_type = void.pointer() # object_
    is_vararg = True

class PyErr_Occurred(ExternalFunction):
    arg_types = []
    return_type = void.pointer() # object_

class PyErr_Clear(ExternalFunction):
    arg_types = []
    return_type = void
#
### Object conversions to native types
#
def create_func(name, restype, argtype, d, check_pyerr_occurred=False):
    class PyLong_FromLong(ExternalFunction):
        arg_types = [argtype]
        return_type = restype

    PyLong_FromLong.__name__ = name
    PyLong_FromLong.check_pyerr_occurred = check_pyerr_occurred

    if restype.is_object:
        type = argtype
    else:
        type = restype

    d[type] = PyLong_FromLong
    globals()[name] = PyLong_FromLong

# The pipeline is using this dictionary to lookup casting func
_as_long = {}
def as_long(name, type):
    create_func(name, type, object_, _as_long, check_pyerr_occurred=True)

as_long('PyLong_AsLong', long_)
as_long('PyLong_AsUnsignedLong', ulong)
as_long('PyLong_AsLongLong', longlong)
as_long('PyLong_AsUnsignedLongLong', ulonglong)
#as_long('PyLong_AsSize_t', size_t) # new in py3k
as_long('PyLong_AsSsize_t', Py_ssize_t)

class PyFloat_FromDouble(ExternalFunction):
    arg_types = [double]
    return_type = object_

class PyComplex_RealAsDouble(ExternalFunction):
    arg_types = [object_]
    return_type = double

class PyComplex_ImagAsDouble(ExternalFunction):
    arg_types = [object_]
    return_type = double

class PyComplex_FromDoubles(ExternalFunction):
    arg_types = [double, double]
    return_type = object_

class PyComplex_FromCComplex(ExternalFunction):
    arg_types = [complex128]
    return_type = object_

if not PY3:
    class PyInt_FromString(ExternalFunction):
        arg_types = [c_string_type, c_string_type.pointer(), int_]
        return_type = object_

class PyLong_FromString(ExternalFunction):
    arg_types = [c_string_type, c_string_type.pointer(), long_]
    return_type = object_

class PyFloat_FromString(ExternalFunction):
    arg_types = [object_, c_string_type.pointer()]
    return_type = object_

class PyBool_FromLong(ExternalFunction):
    arg_types = [long_]
    return_type = object_

#
### Conversion of native types to object
#
# The pipeline is using this dictionary to lookup casting func
_from_long = {}

def from_long(name, type):
    create_func(name, object_, type, _from_long)

from_long('PyLong_FromLong', long_)
from_long('PyLong_FromUnsignedLong', ulong)
from_long('PyLong_FromLongLong', longlong)
from_long('PyLong_FromUnsignedLongLong', ulonglong)
from_long('PyLong_FromSize_t', size_t) # new in 2.6
from_long('PyLong_FromSsize_t', Py_ssize_t)

class PyFloat_AsDouble(ExternalFunction):
    arg_types = [object_]
    return_type = double

class PyComplex_AsCComplex(ExternalFunction):
    arg_types = [object_]
    return_type = complex128

def create_binary_pyfunc(name):
    class PyNumber_BinOp(ExternalFunction):
        arg_types = [object_, object_]
        return_type = object_
    PyNumber_BinOp.__name__ = name
    globals()[name] = PyNumber_BinOp

create_binary_pyfunc('PyNumber_Add')
create_binary_pyfunc('PyNumber_Subtract')
create_binary_pyfunc('PyNumber_Multiply')
if PY3:
    create_binary_pyfunc('PyNumber_TrueDivide')
else:
    create_binary_pyfunc('PyNumber_Divide')
create_binary_pyfunc('PyNumber_Remainder')

class PyNumber_Power(ExternalFunction):
    arg_types = [object_, object_, object_]
    return_type = object_

create_binary_pyfunc('PyNumber_Lshift')
create_binary_pyfunc('PyNumber_Rshift')
create_binary_pyfunc('PyNumber_Or')
create_binary_pyfunc('PyNumber_Xor')
create_binary_pyfunc('PyNumber_And')
create_binary_pyfunc('PyNumber_FloorDivide')

class PyNumber_Positive(ofunc):
    pass

class PyNumber_Negative(ofunc):
    pass

class PyNumber_Invert(ofunc):
    pass

class PyObject_IsTrue(ExternalFunction):
    arg_types = [object_]
    return_type = int_

class PyObject_RichCompareBool(ExternalFunction):
    arg_types = [object_, object_, int_]
    return_type = int_
    badval = -1
    # check_pyerr_occurred = True

class PyObject_RichCompare(ExternalFunction):
    arg_types = [object_, object_, int_]
    return_type = object_

__all__ = [k for k, v in globals().items()
           if (v != ExternalFunction
               and isinstance(v, type)
               and issubclass(v, ExternalFunction))]



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import sys, os.path

_importlist = "VisitorBase parse check".split()

_cd = os.path.join(os.path.dirname(__file__))
version_specific_path = None
common_path = os.path.join(_cd, 'common')

def _get_asdl_depending_on_version():
    '''Export names in the correct asdl.py depending on the python version.
    '''

    global version_specific_path
    major, minor = sys.version_info[0], sys.version_info[1]
    # Assumes that specific-path and common-path are a subdirectory
    # Build an absolute module path.
    prefix = __name__.rsplit('.', 1)
    # The else-case is for running tests in the current directory
    base = (prefix[0] + '.') if len(prefix) > 1 else ''
    dir = 'py%d_%d' % (major, minor)
    version_specific_path = os.path.join(_cd, dir)

    use_abs_import = 0

    modname = base + dir + '.asdl'
    try:
        # try to import from version specific directory
        mod = __import__(modname, fromlist=_importlist, level=use_abs_import)
    except ImportError:
        # fallback to import from common directory
        dir = 'common'
        modname = base + dir + '.asdl'
        mod = __import__(modname, fromlist=_importlist) #, level=use_rel_and_abs)
    for i in _importlist:
        globals()[i] = getattr(mod, i)

def load(filename):
    '''Load ASDL from the version_specific_path if exists,
    or from the generic_path.
    '''
    srcfile = os.path.join(version_specific_path, filename)
    if not os.path.exists(srcfile):
        srcfile = os.path.join(common_path, filename)
    asdl = parse(srcfile)
    assert check(asdl), "Invalid asdl %s" % srcfile
    return asdl

# initialize
_get_asdl_depending_on_version()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba.external import utility

def external_call(context, llvm_module, name, args=(), temp_name=None):
    extfn = context.external_library.get(name)
    return external_call_func(context, llvm_module, extfn, args, temp_name)

def utility_call(context, llvm_module, name, args=(), temp_name=None):
    extfn = context.utility_library.get(name)
    return external_call_func(context, llvm_module, extfn, args, temp_name)

def external_call_func(context, llvm_module, extfn, args=(), temp_name=None):
    '''Build a call node to the specified external function.

    context --- A numba context
    llvm_module --- A LLVM llvm_module
    name --- Name of the external function
    args --- [optional] argument of for the call
    temp_name --- [optional] Name of the temporary value in LLVM IR.
    '''
    from numba import nodes
    temp_name = temp_name or extfn.name
    assert temp_name is not None

    sig = extfn.signature
    lfunc = extfn.declare_lfunc(context, llvm_module)

    exc_check = dict(badval   = extfn.badval,
                     goodval  = extfn.goodval,
                     exc_msg  = extfn.exc_msg,
                     exc_type = extfn.exc_type,
                     exc_args = extfn.exc_args)

    result = nodes.NativeCallNode(sig, args, lfunc, name=temp_name, **exc_check)

    if extfn.check_pyerr_occurred:
        result = nodes.PyErr_OccurredNode(result)

    return result



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import llvm.core

from numba import *
from numba.utility.cbuilder import refcounting

class RefcountingMixin(object):

    def refcount(self, func, value):
        "Refcount a value with a refcounting function"
        assert not self.nopython

        refcounter = self.context.cbuilder_library.declare(func, self.env,
                                                           self.llvm_module)
        object_ltype = object_.to_llvm(self.context)

        b = self.builder
        return b.call(refcounter, [b.bitcast(value, object_ltype)])

    def decref(self, value):
        "Py_DECREF a value"
        return self.refcount(refcounting.Py_DECREF, value)

    def incref(self, value):
        "Py_INCREF a value"
        return self.refcount(refcounting.Py_INCREF, value)

    def xdecref(self, value):
        "Py_XDECREF a value"
        return self.refcount(refcounting.Py_XDECREF, value)

    def xincref(self, value):
        "Py_XINCREF a value"
        return self.refcount(refcounting.Py_XINCREF, value)

    def xdecref_temp(self, temp):
        "Py_XDECREF a temporary"
        return self.xdecref(self.load_tbaa(temp, object_))

    def xincref_temp(self, temp):
        "Py_XINCREF a temporary"
        return self.xincref(self.load_tbaa(temp, object_))

    def xdecref_temp_cleanup(self, temp):
        """
        Cleanup a temp at the end of the function:

            * Save current basic block
            * Generate code at cleanup path
            * Restore basic block
        """
        assert not self.nopython
        bb = self.builder.basic_block

        self.builder.position_at_end(self.current_cleanup_bb)
        self.xdecref_temp(temp)
        self.current_cleanup_bb = self.builder.basic_block

        self.builder.position_at_end(bb)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
import llvm.core
from llvm_cbuilder import builder

from_numba = builder.CStruct.from_numba_struct

class NumbaCDefinition(builder.CDefinition):
    """
    Numba utility simplifying dealing with llvm_cbuilder.
    """

    def __init__(self, env, llvm_module):
        # Environments
        self.env = env
        self.func_env = env.translation.crnt
        self.context = env.context

        self.llvm_module = llvm_module

        self.set_signature(self.env, self.context)

        type(self)._name_ = type(self).__name__
        super(NumbaCDefinition, self).__init__()

    #------------------------------------------------------------------------
    # Convenience Methods
    #------------------------------------------------------------------------

    def external_cfunc(self, func_name):
        "Get a CFunc from an external function"
        signature, lfunc = self.env.context.external_library.declare(
                self.llvm_module,
                func_name)
        assert lfunc.module is self.llvm_module
        return builder.CFunc(self, lfunc)

    def cbuilder_cfunc(self, numba_cdef):
        "Get a CFunc from a NumbaCDefinition"
        lfunc = self.env.context.cbuilder_library.declare(numba_cdef, self.env,
                                                          self.llvm_module)
        assert lfunc.module is self.llvm_module
        return builder.CFunc(self, lfunc)

    #------------------------------------------------------------------------
    # CDefinition stuff
    #------------------------------------------------------------------------

    def signature(self):
        argtypes = [type for name, type in self._argtys_]
        return llvm.core.Type.function(self._retty_, argtypes)

    def __call__(self, module):
        lfunc = super(NumbaCDefinition, self).__call__(module)
        # lfunc.linkage = llvm.core.LINKAGE_LINKONCE_ODR
        return lfunc

    def set_signature(self, env, context):
        """
        Set the cbuilder signature through _argtys_ and optionally the
        _retty_ attributes.
        """



# -*- coding: utf-8 -*-
'''
This module implements constant folding on the AST.  It handles simple cases
such as 
    
* 1 + 2 -> 3
* 2 ** 10 -> 1024
* N=1; N + 1 -> 2  (for N is assigned as global variable or a variable 
                    that's only assigned once)
from __future__ import print_function, division, absolute_import
'''
import operator, ast
from functools import reduce
from . import visitors

# shamelessly copied from Cython
compile_time_binary_operators = {
    '<'     : operator.lt,
    '<='    : operator.le,
    '=='    : operator.eq,
    '!='    : operator.ne,
    '>='    : operator.ge,
    '>'     : operator.gt,
    'is'    : operator.is_,
    'is_not': operator.is_not,
    '+'     : operator.add,
    '&'     : operator.and_,
    '/'     : operator.truediv,
    '//'    : operator.floordiv,
    '<<'    : operator.lshift,
    '%'     : operator.mod,
    '*'     : operator.mul,
    '|'     : operator.or_,
    '**'    : operator.pow,
    '>>'    : operator.rshift,
    '-'     : operator.sub,
    '^'     : operator.xor,
    'in'    : operator.contains,
    'not_in': lambda x, y: not operator.contains(x, y),
    'and'   : operator.and_,
    'or'    : operator.or_,
}

# shamelessly copied from Cython
compile_time_unary_operators = {
    'not'   : operator.not_,
    '~'     : operator.inv,
    '-'     : operator.neg,
    '+'     : operator.pos,
}

ast_to_binary_operator = {
    ast.Add     : '+',
    ast.Sub     : '-',
    ast.Mult    : '*',
    ast.Div     : '/',
    ast.FloorDiv: '//',
    ast.Pow     : '**',
    ast.LShift  : '<<',
    ast.RShift  : '>>',
    ast.BitOr   : '|',
    ast.BitAnd  : '&',
    ast.BitXor  : '^',
    ast.Lt      : '<',
    ast.LtE     : '<=',
    ast.Gt      : '>',
    ast.GtE     : '>=',
    ast.Eq      : '==',
    ast.NotEq   : '!=',
    ast.Is      : 'is',
    ast.IsNot   : 'is_not',
    ast.In      : 'in',
    ast.NotIn   : 'not_in',
    ast.And     : 'and',
    ast.Or      : 'or',
}

ast_to_unary_operator = {
    ast.Not     : 'not',
    ast.Invert  : '~',
    ast.USub    : '-',
    ast.UAdd    : '+',
}

class NotConstExprError(Exception):
    pass

class ConstantExprRecognizer(ast.NodeVisitor):
    def __init__(self, const_name_set):
        self.const_name_set = const_name_set

    def visit_BinOp(self, node):
        if not(self.visit(node.left) and self.visit(node.right)):
            raise NotConstExprError

    def visit_BoolOp(self, node):
        if not all(self.visit(x) for x in node.values):
            raise NotConstExprError

    def visit_Compare(self, node):
        if not(node.left and all(self.visit(x) for x in node.comparators)):
            raise NotConstExprError

    def generic_visit(self, node):
        if not is_constant(node, self.const_name_set):
            raise NotConstExprError

    def __call__(self, node):
        try:
            self.visit(node)
        except NotConstExprError as e:
            return False
        else:
            return True

class ConstantMarker(visitors.NumbaVisitor):
    '''A conservative constant marker.  Conservative because we handle the
    simplest cases only.
    '''
    def __init__(self, *args, **kws):
        super(ConstantMarker, self).__init__(*args, **kws)
        self._candidates = {}  # variable name -> value (rhs) node
        self._invalidated = set()

    def visit_Assign(self, node):
        targets = []
        for target in node.targets:
            targets.extend(self._flatten_aggregate(target))

        # targets contains tuple/list
        not_handled = len(node.targets) != len(targets)
                
        for target in targets:
            try:
                name = target.id
            except AttributeError:
                # Only for assignment into simple name on the LHS
                pass
            else:
                if name not in self._invalidated:
                    if not_handled or name in self._candidates:
                        self._invalidate(name)
                    else:
                        self._candidates[name] = node.value

    def visit_AugAssign(self, node):
        try:
            name = node.target.id
        except AttributeError:
            # Only for assignment into simple name on the LHS
            pass
        else:
            if name in self._candidates:
                self._invalidate(name)

    def visit_For(self, node):
        targets = self._flatten_aggregate(node.target)
        for t in targets:
            self._invalidate(t.id)
        for instr in node.body:
            self.visit(instr)

    def _flatten_aggregate(self, node):
        assert isinstance(node.ctx, ast.Store)
        if isinstance(node, ast.Tuple) or isinstance(node, ast.List):
            ret = []
            for i in node.elts:
                ret.extend(self._flatten_aggregate(i))
            return ret
        else:
            return [node]

    def _invalidate(self, name):
        self._invalidated.add(name)
        try:
            del self._candidates[name]
        except KeyError:
            pass

    def get_constants(self):
        '''Return a set of constant variable names
        '''
        const_names = set(self.varnames).difference(self._invalidated)
        const_names |= set(self.func_globals)

        constexpr_recognizer = ConstantExprRecognizer(const_names)
        retvals = []
        for k, v in self._candidates.items():
            if constexpr_recognizer(v):
                retvals.append(k)
        return set(retvals)

class ConstantFolder(visitors.NumbaTransformer):
    '''Perform constant folding on AST.

    NOTE: Forbids assignment to True, False, None.
    '''
    def __init__(self, *args, **kws):
        assert not hasattr(self, 'constvars') # not overwriting
        assert not hasattr(self, 'constvalues') # not overwriting
        self.constvars = kws.pop('constvars')
        self.constvalues = {}
        super(ConstantFolder, self).__init__(*args, **kws)

    def visit_BinOp(self, node):
        lval = node.left = self.visit(node.left)
        rval = node.right = self.visit(node.right)

        if self.is_constant(lval) and self.is_constant(rval):
            return self.eval_binary_operation(node.op, lval, rval)
        else:
            return node

    def visit_BoolOp(self, node):
        values = node.values = [self.visit(nd) for nd in node.values]
        if all(self.is_constant(v) for v in values):
            operation = lambda x, y: self.eval_binary_operation(node.op, x, y)
            return reduce(operation, values)
        else:
            return node

    def visit_Compare(self, node):
        left = node.left = self.visit(node.left)
        comparators = node.comparators = [self.visit(nd)
                                          for nd in node.comparators]
        operands = [left] + comparators
        operators = iter(reversed(node.ops))

        def operation(x, y):
            op = next(operators)
            return self.eval_binary_operation(op, x, y)

        if all(self.is_constant(nd) for nd in operands):
            return reduce(operation, operands)
        else:
            return node

    def visit_Assign(self, node):
        '''Store the rhs value so we can inline them in future reference.
            
        TODO: Remove assignment of constant
        '''
        # FIXME: does not handle assign to tuple
        names = []
        value = node.value = self.visit(node.value)
        for left in node.targets:
            try:
                name = left.id
            except AttributeError:
                return node # escape
            else:
                names.append(name)

        ct = 0
        for name in names:
            if name in self.constvars: # is known constant
                assert name not in self.constvalues
                self.constvalues[name] = value
                self.constvars.remove(name)
                ct += 1

        return node

    def visit_Name(self, node):
        if isinstance(node.ctx, ast.Load) and self.is_constant(node):
            try:
                return self.constvalues[node.id]
            except KeyError:
                val = self.func_globals.get(node.id)
                if val and is_simple_value(val):
                    if isinstance(val, (int, long, float)):
                        return ast.Num(n=val)
                    elif isinstance(val, bool):
                        name = 'True' if val else 'False'
                        return ast.Name(id=name, ctx=ast.Load())
        return node

    def eval_binary_operation(self, op, left, right):
        '''Evaluate the constant expression and return a ast.Num instance
        containing the result.
        '''
        operator = ast_to_binary_operator[type(op)]
        func = compile_time_binary_operators[operator]
        ret = func(self.valueof(left), self.valueof(right))
        if ret is True:
            node = ast.Name(id='True', ctx=ast.Load())
        elif ret is False:
            node = ast.Name(id='False', ctx=ast.Load())
        elif ret is None:
            node = ast.Name(id='None', ctx=ast.Load())
        else:
            node = ast.Num(n=ret)
        return ast.copy_location(node, left)
    
    def valueof(self, node):
        '''Get constant value from AST node.
        '''
        if isinstance(node, ast.Num):
            return node.n
        elif isinstance(node, ast.Name) and isinstance(node.ctx, ast.Load):
            if node.id == 'True':
                return True
            elif node.id == 'False':
                return False
            elif node.id == 'None':
                return None
            elif node.id in self.constvalues:
                return self.valueof(self.constvalues[node.id])
            else:
                value = self.func_globals[node.id]
                if not is_simple_value(value):
                    raise ValueError("%s is not a simple value.")
                return value
        raise ValueError("node %s is not a has constant value" % node)

    def is_constant(self, node):
        globals = set(self.func_globals).difference(self.local_names)
        return is_constant(node, globals | set(self.constvalues))

def is_constant(node, constants=set()):
    if isinstance(node, ast.Num):
        return True
    elif isinstance(node, ast.Name) and isinstance(node.ctx, ast.Load):
        if node.id in ['True', 'False', 'None']:
            return True
        elif node.id in constants:
            return True
    return False

def is_simple_value(value):
    return (   isinstance(value, int)
            or isinstance(value, long)
            or isinstance(value, float)
            or value is True
            or value is False
            or value is None)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import
from numba import nodes
from numba.reporting import getpos

class StatementDescr(object):
    is_assignment = False

class LoopDescr(object):
    def __init__(self, next_block, loop_block):
        self.next_block = next_block
        self.loop_block = loop_block
        self.exceptions = []


class ExceptionDescr(object):
    """Exception handling helper.

    entry_point   ControlBlock Exception handling entry point
    finally_enter ControlBlock Normal finally clause entry point
    finally_exit  ControlBlock Normal finally clause exit point
    """

    def __init__(self, entry_point, finally_enter=None, finally_exit=None):
        self.entry_point = entry_point
        self.finally_enter = finally_enter
        self.finally_exit = finally_exit


class NameAssignment(object):

    is_assignment = True

    def __init__(self, lhs, rhs, entry, assignment_node, warn_unused=True):
        if not hasattr(lhs, 'cf_state'):
            lhs.cf_state = set()
        if not hasattr(lhs, 'cf_is_null'):
            lhs.cf_is_null = False

        self.lhs = lhs
        self.rhs = rhs
        self.assignment_node = assignment_node

        self.entry = entry
        self.pos = getpos(lhs)
        self.refs = set()
        self.is_arg = False
        self.is_deletion = False

        # NOTE: this is imperfect, since it means warnings are disabled for
        # *all* definitions in the function...
        self.entry.warn_unused = warn_unused

    def __repr__(self):
        return '%s(entry=%r)' % (self.__class__.__name__, self.entry)

    def infer_type(self, scope):
        return self.rhs.infer_type(scope)

    def type_dependencies(self, scope):
        return self.rhs.type_dependencies(scope)

class AttributeAssignment(object):
    """
    Assignment to some attribute. We need to detect assignments in the
    constructor of extension types.
    """

    def __init__(self, assmnt):
        self.assignment_node = assmnt
        self.lhs = assmnt.targets[0]
        self.rhs = assmnt.value

class Argument(NameAssignment):
    def __init__(self, lhs, rhs, entry):
        NameAssignment.__init__(self, lhs, rhs, entry)
        self.is_arg = True


class PhiNode(nodes.Node):

    def __init__(self, block, variable):
        self.block = block
        # Unrenamed variable. This will be replaced by the renamed version
        self.variable = variable
        self.type = None
        # self.incoming_blocks = []

        # Set of incoming variables
        self.incoming = set()
        self.phis = set()

        self.assignment_node = self

    @property
    def entry(self):
        return self.variable

    def add_incoming_block(self, block):
        self.incoming_blocks.append(block)

    def add(self, block, assmnt):
        if assmnt is not self:
            self.phis.add((block, assmnt))

    def __repr__(self):
        lhs = self.variable.name
        if self.variable.renamed_name:
            lhs = self.variable.unmangled_name
        incoming = ", ".join("var(%s, %s)" % (var_in.unmangled_name, var_in.type)
            for var_in in self.incoming)
        if self.variable.type:
            type = str(self.variable.type)
        else:
            type = ""
        return "%s %s = phi(%s)" % (type, lhs, incoming)

    def find_incoming(self):
        for parent_block in self.block.parents:
            name = self.variable.name
            incoming_var = parent_block.symtab.lookup_most_recent(name)
            yield parent_block, incoming_var


class NameDeletion(NameAssignment):
    def __init__(self, lhs, entry):
        NameAssignment.__init__(self, lhs, lhs, entry)
        self.is_deletion = True

class Uninitialized(object):
    pass

class NameReference(object):
    def __init__(self, node, entry):
        if not hasattr(node, 'cf_state'):
            node.cf_state = set()
        self.node = node
        self.entry = entry
        self.pos = getpos(node)

    def __repr__(self):
        return '%s(entry=%r)' % (self.__class__.__name__, self.entry)



# -*- coding: utf-8 -*-
"""
Infer types for NumPy functionality. This includes:

    1) Figuring out dtypes

        e.g. np.double     -> double
             np.dtype('d') -> double

    2) Function calls such as np.empty/np.empty_like/np.arange/etc
"""
from __future__ import print_function, division, absolute_import

import warnings
from functools import reduce

import numpy as np

from numba import *
from numba.minivect import minitypes
from numba import typesystem, error
from numba.type_inference.module_type_inference import (register,
                                                        register_inferer,
                                                        register_unbound)
from numba.typesystem import get_type


#------------------------------------------------------------------------
# Type Definitions
#------------------------------------------------------------------------

index_array_t = npy_intp[:]

#------------------------------------------------------------------------
# Some utilities
#------------------------------------------------------------------------

def promote(context, *types):
    return reduce(context.promote_types, map(array_from_type, types))

def resolve_attribute_dtype(dtype, default=None):
    "Resolve the type for numpy dtype attributes"
    if dtype.is_numpy_dtype:
        return dtype

    if dtype.is_known_value:
        numpy_attr = dtype.value
        if isinstance(numpy_attr, np.dtype):
            return typesystem.from_numpy_dtype(numpy_attr)
        elif issubclass(numpy_attr, np.generic):
            return typesystem.from_numpy_dtype(np.dtype(numpy_attr))
        elif numpy_attr is not None:
            try:
                dtype = np.dtype(numpy_attr)
            except TypeError:
                warnings.warn("Unable to infer dtype for '%s'" % numpy_attr)
            else:
                return typesystem.from_numpy_dtype(dtype)

    return None

def get_dtype(dtype_arg, default_dtype=None):
    """
    Simple helper function to map an AST node dtype keyword
    argument => NumPy dtype.
    '"""
    if dtype_arg is None:
        if default_dtype is None:
            return None

        return typesystem.dtype(default_dtype)
    else:
        return resolve_attribute_dtype(dtype_arg)

def promote_to_array(dtype):
    "Promote scalar to 0d array type"
    if not dtype.is_array:
        dtype = minitypes.ArrayType(dtype, 0)
    return dtype

def demote_to_scalar(type):
    "Demote 0d arrays to scalars"
    if type and type.is_array and type.ndim == 0:
        return type.dtype
    return type

def array_from_object(a):
    """
    object -> array type:

        array_from_object(ASTNode([[1, 2], [3, 4]])) => int64[:, :]
    """
    return array_from_type(get_type(a))

def array_from_type(type):
    if type.is_array:
        return type
    elif type.is_tuple or type.is_list:
        dtype = array_from_type(type.base_type)
        if dtype.is_array:
            type = dtype.copy()
            type.ndim += 1
            return type
    elif not type.is_object:
        return minitypes.ArrayType(dtype=type, ndim=0)

    return object_

#------------------------------------------------------------------------
# Resolution of NumPy calls
#------------------------------------------------------------------------

@register(np)
def dtype(obj, align):
    "Parse np.dtype(...) calls"
    if obj is None:
        return None

    return get_dtype(obj)

def empty_like(a, dtype, order):
    "Parse the result type for np.empty_like calls"
    if a is None:
        return None

    if a.is_array:
        if dtype:
            dtype_type = get_dtype(dtype)
            if dtype_type is None:
                return a
            dtype = dtype_type.dtype
        else:
            dtype = a.dtype

        return typesystem.array(dtype, a.ndim)

register_inferer(np, 'empty_like', empty_like)
register_inferer(np, 'zeros_like', empty_like)
register_inferer(np, 'ones_like', empty_like)

def empty(shape, dtype, order):
    if shape is None:
        return None

    dtype = get_dtype(dtype, float64)
    if dtype is None:
        return object_

    if shape.is_int:
        ndim = 1
    elif shape.is_tuple or shape.is_list:
        ndim = shape.size
    else:
        return None

    return typesystem.array(dtype.dtype, ndim)

register_inferer(np, 'empty', empty)
register_inferer(np, 'zeros', empty)
register_inferer(np, 'ones', empty)

@register(np)
def arange(start, stop, step, dtype):
    "Resolve a call to np.arange()"
    # NOTE: dtype must be passed as a keyword argument, or as the fourth
    # parameter
    dtype_type = get_dtype(dtype, npy_intp)
    if dtype_type is not None:
        # return a 1D array type of the given dtype
        return dtype_type.dtype[:]

@register(np)
def dot(context, a, b, out):
    "Resolve a call to np.dot()"
    if out is not None:
        return out

    lhs_type = promote_to_array(a)
    rhs_type = promote_to_array(b)

    dtype = context.promote_types(lhs_type.dtype, rhs_type.dtype)
    dst_ndim = lhs_type.ndim + rhs_type.ndim - 2

    result_type = typesystem.array(dtype, dst_ndim)
    return result_type

@register(np)
def array(object, dtype, order, subok):
    type = array_from_type(object)
    if type.is_array and dtype is not None:
        type = type.copy(dtype=dtype)
    elif dtype is not None:
        return dtype
    else:
        return type

@register(np)
def nonzero(a):
    return _nonzero(array_from_type(a))

def _nonzero(type):
    if type.is_array:
        return typesystem.TupleType(index_array_t, type.ndim)
    else:
        return typesystem.TupleType(index_array_t)

@register(np)
def where(context, condition, x, y):
    if x is None and y is None:
        return nonzero(condition)

    return promote(context, x, y)

@register(np)
def vdot(context, a, b):
    lhs_type = promote_to_array(a)
    rhs_type = promote_to_array(b)
    dtype = context.promote_types(lhs_type.dtype, rhs_type.dtype)
    return dtype

@register(np)
def inner(context, a, b):
    lhs_type = promote_to_array(a)
    rhs_type = promote_to_array(b)
    dtype = context.promote_types(lhs_type.dtype, rhs_type.dtype)
    if lhs_type.ndim == 0:
        result_ndim = rhs_type.ndim
    elif rhs_type.ndim == 0:
        result_ndim = lhs_type.ndim
    else:
        result_ndim = lhs_type.ndim + rhs_type.ndim - 2
    if result_ndim == 0:
        result_type = dtype
    else:
        result_type = typesystem.array(dtype, result_ndim)
    return result_type

@register(np)
def outer(context, a, b):
    result_type = promote(context, a, b)
    # promote() converts scalar types to 0-dim arrays, so it should
    # always return an array type.  Ensure this continues to hold...
    assert result_type.is_array
    return result_type.dtype[:, :]

@register(np, pass_in_types=False)
def tensordot(context, a, b, axes):
    '''Typing function for numpy.tensordot().

    Defaults to Python object for any caller that isn't using the
    default argument to axes.

    Otherwise, it is similar to inner(), but subtracts four dimensions
    from the result instead of two.

    Without symbolic execution of the actual axes argument, this can't
    determine the number of axes to sum over, so it punts.  This
    typing function could use an array type of unknown dimensionality,
    were one available.  See:
    https://www.pivotaltracker.com/story/show/43687249
    '''
    lhs_type = array_from_object(a)
    rhs_type = array_from_object(b)
    if lhs_type.ndim < 1:
        raise error.NumbaError(a, 'First argument to numpy.tensordot() '
                               'requires array of dimensionality >= 1.')
    elif rhs_type.ndim < 1:
        raise error.NumbaError(b, 'First argument to numpy.tensordot() '
                               'requires array of dimensionality >= 1.')
    dtype = context.promote_types(lhs_type.dtype, rhs_type.dtype)
    if axes is None:
        result_ndim = lhs_type.ndim + rhs_type.ndim - 4
        if result_ndim < 0:
            raise error.NumbaError(a, 'Arguments to numpy.tensordot() should '
                                   'have combined dimensionality >= 4 (when '
                                   'axes argument is not specified).')
        result_type = typesystem.array(dtype, result_ndim)
    else:
        # XXX Issue warning to user?
        result_type = object_
    return result_type

@register(np)
def einsum(context, subs, operands, kws):
    # XXX Issue warning to user?
    # XXX Attempt type inference in case where subs is a string?
    return object_

@register(np)
def kron(context, a, b):
    #raise NotImplementedError("XXX")
    return object_

@register(np)
def trace(context, a, offset, axis1, axis2, dtype, out):
    #raise NotImplementedError("XXX")
    return object_

#------------------------------------------------------------------------
# numpy.linalg
#------------------------------------------------------------------------

@register(np.linalg)
def cholesky(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def cond(context, x, p):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def det(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def eig(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def eigh(context, a, UPLO):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def eigvals(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def eigvalsh(context, a, UPLO):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def inv(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def lstsq(context, a, b, rcond):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def matrix_power(context, M, n):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def matrix_rank(context, M, tol):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def norm(context, x, ord):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def pinv(context, a, rcond):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def qr(context, a, mode):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def slogdet(context, a):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def solve(context, a, b):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def svd(context, a, full_matrices, compute_uv):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def tensorinv(context, a, ind):
    #raise NotImplementedError("XXX")
    return object_

@register(np.linalg)
def tensorsolve(context, a, b, axes):
    #raise NotImplementedError("XXX")
    return object_
